@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: cmd_copy.c,v 1.86 2022/03/26 22:17:10 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: cmd_copy.c,v 1.109 2025/01/18 19:37:45 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -88,52 +88,52 @@ pgoblin_copyout(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         goto out;
     }
     if  (!e) {
-        MARK_R_CONN_GO(rn[PGO_CCON]);;
+        MARK_R_CONN_GO(rdb);;
         ;;  rdb->flags &= ~PGOBLIN_ACR_MASK;
         ;;  rdb->flags |= PGOBLIN_CR_OUT;
-        MARK_R_CONN_WENT(rn[PGO_CCON]);;
-        e = pgoblin_db_query(exenv, rn[PGO_CCON], rn[PGO_COUT], rn[PGO_CCTL]);
+        MARK_R_CONN_WENT(rdb);;
+        e = pgoblin_db_query(exenv, rn);
         if  (e) {
             e = -1;
             goto out;
         }
-        for (; (f = pgoblin_db_getcopy(exenv, rn[PGO_CCON], &buf)) > 0 && e >= 0;) {
+        for (; (f = pgoblin_db_getcopy(exenv, rn, &buf)) > 0 && e >= 0;) {
             ifBLIN_QX2("%d buf=%s", f, (char*)buf);
             if  (strcmp("\\.", buf) == 0) {
                 if  (exenv->options->flags & PGOBLIN_COPEND_PASS)
-                    e = pgoblin_wr(rou, buf, f);
+                    e = pgoblin_io_write(rou, buf, (size_t)f);
                 free(buf);
                 break;
             }
-            e = pgoblin_wr(rou, buf, f);
+            e = pgoblin_io_write(rou, buf, (size_t)f);
             free(buf);
         }
         if  ((e < 0) || (f < 0)) {
             ifBLIN_QX0("getcopy error");
             e = f;
         }
-        MARK_IO_PQ_GO(rn[PGO_COUT]);;
-        ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-        MARK_IO_PQ_WENT(rn[PGO_COUT]);;
-        if  (pgoblin_db_endstream(exenv, rn[PGO_CCON])) {
+        MARK_IO_PQ_GO(rou);;
+        ;;  pgoblin_db_clear(rou);
+        MARK_IO_PQ_WENT(rou);;
+        if  (pgoblin_db_endstream(exenv, rn)) {
             e = -1;
             goto out;
     }   }
     if  (e < 0) {
         ifBLIN_QX0("write error");
     } else if (exenv->options->flags & PGOBLIN_COPEND_APND) {
-        e = pgoblin_wr(rou, "\\.\n", 0);
+        e = pgoblin_io_write(rou, "\\.\n", 0);
     }
     if  (e < 0) {
         ifBLIN_QX0("write end error");
     }
-    MARK_IO_PQ_GO(rn[PGO_COUT]);;
-    ;;  pgoblin_db_clear(exenv, rn[PGO_COUT]);
-    MARK_IO_PQ_WENT(rn[PGO_COUT]);;
+    MARK_IO_PQ_GO(rou);;
+    ;;  pgoblin_db_clear(rou);
+    MARK_IO_PQ_WENT(rou);;
 out:
-    MARK_R_CONN_GO(rn[PGO_CCON]);;
+    MARK_R_CONN_GO(rdb);;
     ;;  rdb->flags &= ~PGOBLIN_ACR_MASK;
-    MARK_R_CONN_WENT(rn[PGO_CCON]);;
+    MARK_R_CONN_WENT(rdb);;
     ifBLIN_QX3("- %"BLIN_D, e);
     return(e < 0 ? (int)e : 0);
 #   undef blin_internal_flags
@@ -155,7 +155,6 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
     int           ee;                                                /*   putcopy */
     int           ex = 0;                                             /*   putcopy */
     ssize_t       f  = 0;
-    char         *d;
     char         *q;
     enum finde    k;
                                                    /*    "\n\\.\n"    */
@@ -191,7 +190,7 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     if  (!rn[PGO_COUT]) {
         nou = PGOBLIN_REGSIZE - 1;
-        if  (!(rou = pgoblin_pushio(exenv->options, nou))) {
+        if  (!(rou = pgoblin_pushio(exenv, nou))) {
            ifBLIN_QW0("No IO reg -");
            ex = -EX_CANTCREAT;
            goto out;
@@ -209,26 +208,25 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
     GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     GET_RIO(rct, exenv->options, rn[PGO_CCTL]);
     GET_RIO(rin, exenv->options, rn[PGO_CIN]);
-    d = NULL;                                                                  /*   COPY */
     if  (!ex && !rct->text) {
         ifBLIN_QX0("No query");
-        ex = -1;
+        goto brk;
     }
     if  (!rin->mife) {
-        ifBLIN_QX0("No input data");
-        ex = -1;
+        ifBLIN_QX2("No input data");
+        goto brk;
     }
     if  (!ex) {
-        MARK_R_CONN_GO(rn[PGO_CCON]);;
+        MARK_R_CONN_GO(rdb);;
         ;;  rdb->flags &= ~PGOBLIN_ACR_MASK;
         ;;  rdb->flags |= PGOBLIN_CR_IN;
-        MARK_R_CONN_WENT(rn[PGO_CCON]);;
-        if  (!!(ex = pgoblin_db_query(exenv, rn[PGO_CCON], rn[PGO_COUT], rn[PGO_CCTL]))) {
+        MARK_R_CONN_WENT(rdb);;
+        if  (!!(ex = pgoblin_db_query(exenv, rn))) {
             ifBLIN_QW0("Error query=%s~", rct->text);
             ex = -1;
             goto out;
         }
-/*      if  (0 > (ex = pgoblin_db_resinfo(exenv, rn[PGO_COUT], PGOBLIN_Nfields))) {
+/*      if  (0 > (ex = pgoblin_db_resinfo(rn[PGO_COUT], PGOBLIN_Nfields))) {
  *          ifBLIN_QW0("Error Nfields in query=%s~", rct->text);
  *          ex = -1;
  *          goto out;
@@ -236,19 +234,19 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
  */
         ifBLIN_QX3("Copy %d columns @%"BLIN_X" + %"BLIN_X, ex, (size_t)mife_offset(rin->mife), f);
         ex = 0;
-        MARK_IO_PQ_GO(nou);;
-        ;;  pgoblin_db_clear(exenv, nou);
-        MARK_IO_PQ_WENT(nou);;
+        MARK_IO_PQ_GO(rou);;
+        ;;  pgoblin_db_clear(rou);
+        MARK_IO_PQ_WENT(rou);;
         k = finde_eol;
         do {
             char *ofbase;
 
-            if  ((f = mife_read(rin->mife, PGOBLIN_BUFSIZE, mife_offset(rin->mife) + f)) < 0) {
-                ifBLIN_QW0("mife_read");
+            if  ((f = pgoblin_io_read(rin, PGOBLIN_BUFSIZE, mife_offset(rin->mife) + f)) < 0) {
+                ifBLIN_QW0("pgoblin_io_read");
                 ex = -1;
             }
-            ifBLIN_QX4("mife_read=%"BLIN_D, BLIN_I(f));
-            ofbase = mife_get(rin->mife, mife_offset(rin->mife));
+            ifBLIN_QX4("pgoblin_io_read=%"BLIN_D, BLIN_I(f));
+            ofbase = mife_pointer(rin->mife);
             if  (!ex) {
                 for ( q = ofbase
                     ; q < (ofbase + f) && k != finde_end
@@ -257,7 +255,7 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
                     ifBLIN_QX4("%c", *q);
                 }
                 if  (q - ofbase) {
-                    ex = (pgoblin_db_putcopy(exenv, rn[PGO_CCON], ofbase, q - ofbase) < 1);
+                    ex = (pgoblin_db_putcopy(exenv, rn, ofbase, q - ofbase) < 1);
                     ifBLIN_QX3("putcopy=%d %d (%"BLIN_D")", ex, k, BLIN_I(q - ofbase));
                 } else {
                     ifBLIN_QX3("no putcopy=%d %d (%"BLIN_D")", ex, k, BLIN_I(q - ofbase));
@@ -266,30 +264,31 @@ pgoblin_copyin(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
         if  (!ex && k != finde_end) {                      /*  ()    */
             const char *enstin = "\n\\.\n";
             ifBLIN_QX2("end=%s~%d", &enstin[k], finde_end - k);
-            ex = (pgoblin_db_putcopy(exenv, rn[PGO_CCON], &enstin[k], finde_end - k) < 1);
+            ex = (pgoblin_db_putcopy(exenv, rn, &enstin[k], (ssize_t)(finde_end - k)) < 1);
             ifBLIN_QX3("putcopy=%d %d (%d)", ex, k, finde_end - k);
         }
         if  (ex) {                                              /*       */
             ifBLIN_QX0("putcopy failed");
             ex = -1;
         }
-        ee = pgoblin_db_endstream(exenv, rn[PGO_CCON]);
+        ee = pgoblin_db_endstream(exenv, rn);
         if  (!ex && (ee < 0)) {                                 /*       */
             ex = -1;
             ifBLIN_QX0("endcopy failed");
     }   }
-    MARK_IO_PQ_GO(nou);;
-    ;;  pgoblin_db_clear(exenv, nou);
-    MARK_IO_PQ_WENT(nou);;
-    MARK_R_CONN_GO(rn[PGO_CCON]);;
+brk:
+    MARK_IO_PQ_GO(rou);;
+    ;;  pgoblin_db_clear(rou);
+    MARK_IO_PQ_WENT(rou);;
+    MARK_R_CONN_GO(rdb);;
     ;;  rdb->flags &= ~PGOBLIN_ACR_MASK;
-    MARK_R_CONN_WENT(rn[PGO_CCON]);;
+    MARK_R_CONN_WENT(rdb);;
 out:
     if  (nou == (PGOBLIN_REGSIZE - 1)) {
-        MARK_IO_PQ_GO(nou);;
-        ;;  pgoblin_db_clear(exenv, nou);
-        ;;  pgoblin_popio(exenv->options, PGOBLIN_REGSIZE - 1);
-        MARK_IO_PQ_WENT(nou);;
+        MARK_IO_PQ_GO(rou);;
+        ;;  pgoblin_db_clear(rou);
+        ;;  pgoblin_popio(exenv->options, nou);
+        MARK_IO_PQ_WENT(rou);;
     }
     ifBLIN_QX3("- %d", ex);
     return(ex);
@@ -329,7 +328,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-    int          coln;
+    int64_t      coln;
     char        *buf;
     pgoblin_rio *rou;
     pgoblin_rio *rin;
@@ -347,35 +346,35 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         e = -1;
         goto out;
     }
-    coln = pgoblin_db_resinfo(exenv, rn[PGO_CIN], PGOBLIN_Nfields);
-    for (int i = 0; i < coln; ++i) {
+    coln = pgoblin_db_resinfo(rin, PGOBLIN_Nfields);
+    for (int64_t i = 0; i < coln; ++i) {
         const char *cn;
         const char *ct;
         ssize_t     st;
         ssize_t     sz;
-        int         l;
+        ssize_t     l;
         int         k;
 
         l = 2 + (3 * 4 /* 4 fields for future*/);
-        l += nulen(i);
-        cn = pgoblin_db_subinfo(exenv, rn[PGO_CIN], PGOBLIN_ColName, i);
-        ct = pgoblin_db_subinfo(exenv, rn[PGO_CIN], PGOBLIN_TypeNm, i);
-        st = pgoblin_db_valinfo(exenv, rn[PGO_CIN], PGOBLIN_TypeMod, 0, i);
-        sz = pgoblin_db_valinfo(exenv, rn[PGO_CIN], PGOBLIN_ColSize, 0, i);
+        l += nulen((u_int64_t)i);
+        cn = pgoblin_db_subinfo(rin, PGOBLIN_ColName, (int64_t)i);
+        ct = pgoblin_db_subinfo(rin, PGOBLIN_TypeNm, (int64_t)i);
+        st = pgoblin_db_valinfo(rin, PGOBLIN_TypeMod, 0, (ssize_t)i);
+        sz = pgoblin_db_valinfo(rin, PGOBLIN_ColSize, 0, (ssize_t)i);
         if  (cn) l += strlen(cn) * 2 + 1; else l += 3;
         if  (ct) l += strlen(ct) * 2 + 1; else l += 3;
         if  (0 <= st) {
-            l += nulen(st);
+            l += nulen((u_int64_t)st);
         } else {
-            if (PGOBLIN_SSIZE_MIN != st) l += nulen(-st) + 1; else l += 3;
+            if (PGOBLIN_SSIZE_MIN != st) l += nulen((u_int64_t)-st) + 1; else l += 3;
         }
-        if  (0 <= sz) l += nulen(sz); else l += 3;
-        if  (!(buf = malloc(l))) {
+        if  (0 <= sz) l += nulen((u_int64_t)sz); else l += 3;
+        if  (!(buf = malloc((size_t)l))) {
             ifBLIN_QW0("no RAM");
             e = -EX_OSERR;
             goto out;
         }
-        k = snprintf(buf, l, "%d", i);
+        k = snprintf(buf, (size_t)l, "%"BLIN_O"u", i);
         l -= k;
         buf[k++] = '\t';
         l--;
@@ -392,7 +391,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         l -= e;
         k += e;
         if  (PGOBLIN_SSIZE_MIN != st) {
-            e = snprintf(&buf[k], l, "%ld", st);
+            e = snprintf(&buf[k], (size_t)l, "%zd", st);
             l -= e;
             k += e;
             buf[k++] = '\t';
@@ -405,7 +404,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
             k += e;
         }
         if  (0 <= sz) {
-            e = snprintf(&buf[k], l, "%ld", sz);
+            e = snprintf(&buf[k], (size_t)l, "%zd", sz);
             l -= e;
             k += e;
             buf[k++] = '\t';
@@ -423,7 +422,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         }
         l -= e;
         k += e;
-        if  (0 > (e = pgoblin_wr(rou, buf, k))) {
+        if  (0 > (e = pgoblin_io_write(rou, buf, (size_t)k))) {
             ifBLIN_QW0("write error");
             free(buf);
             goto out;
@@ -431,7 +430,7 @@ pgoblin_getmeta(pgoblin_exenv *exenv, pgoblin_nr *rn) {             /**
         free(buf);
     }
     if  (exenv->options->flags & PGOBLIN_COPEND_APND) {
-        if  (0 > (e = pgoblin_wr(rou, "\\.\n", 3))) ifBLIN_QW0("write end error");
+        if  (0 > (e = pgoblin_io_write(rou, "\\.\n", 3))) ifBLIN_QW0("write end error");
     }
 out:
     ifBLIN_QX3("- %"BLIN_D, e);
@@ -466,43 +465,187 @@ pgoblin_info(pgoblin_exenv *exenv, pgoblin_nr *rn) {                /**
         ex = EX_USAGE;
         goto out;
     }
-    ibuf[0] = pgoblin_db_resinfo(exenv, rn[PGO_CIN], PGOBLIN_Ntuples);
-    ibuf[1] = pgoblin_db_resinfo(exenv, rn[PGO_CIN], PGOBLIN_Nfields);
-    ibuf[2] = pgoblin_db_resinfo(exenv, rn[PGO_CIN], PGOBLIN_Nchange);
-    ibuf[3] = pgoblin_db_resinfo(exenv, rn[PGO_CIN], PGOBLIN_LastOid);
-    if  ((0 <= ibuf[0]) && (0 > asprintf(&buf[0], "%ld", ibuf[0]))) {
+    ibuf[0] = pgoblin_db_resinfo(rin, PGOBLIN_Ntuples);
+    ibuf[1] = pgoblin_db_resinfo(rin, PGOBLIN_Nfields);
+    ibuf[2] = pgoblin_db_resinfo(rin, PGOBLIN_Nchange);
+    ibuf[3] = pgoblin_db_resinfo(rin, PGOBLIN_LastOid);
+    if  ((0 <= ibuf[0]) && (0 > asprintf(&buf[0], "%"BLIN_O"d", ibuf[0]))) {
         ifBLIN_QW0("asprintf");
         ex = EX_USAGE;
         goto out;
     }
-    if  ((0 <= ibuf[1]) && (0 > asprintf(&buf[1], "%ld", ibuf[1]))) {
+    if  ((0 <= ibuf[1]) && (0 > asprintf(&buf[1], "%"BLIN_O"d", ibuf[1]))) {
         ifBLIN_QW0("asprintf");
         ex = EX_USAGE;
         goto out;
     }
-    if  ((0 <= ibuf[2]) && (0 > asprintf(&buf[2], "%ld", ibuf[2]))) {
+    if  ((0 <= ibuf[2]) && (0 > asprintf(&buf[2], "%"BLIN_O"d", ibuf[2]))) {
         ifBLIN_QW0("asprintf");
         ex = EX_USAGE;
         goto out;
     }
-    if  ((0 <= ibuf[3]) && (0 > asprintf(&buf[3], "%ld", ibuf[3]))) {
+    if  ((0 <= ibuf[3]) && (0 > asprintf(&buf[3], "%"BLIN_O"d", ibuf[3]))) {
         ifBLIN_QW0("asprintf");
         ex = EX_USAGE;
         goto out;
     }
     for (int i = 0; i < 4; ++i) cuf[i] = buf[i];
-    cuf[6] = pgoblin_db_subinfo(exenv, rn[PGO_CIN], PGOBLIN_CmdStat, 0);
-    MARK_IO_PQ_GO(rn[PGO_COUT]);;
-    ;;  if  (!!(ex = pgoblin_db0_create(exenv, rn[PGO_COUT], 7, cuf, NULL))) {
+    cuf[6] = pgoblin_db_subinfo(rin, PGOBLIN_CmdStat, 0);
+    MARK_IO_PQ_GO(rou);;
+    ;;  if  (!!(ex = pgoblin_db0_create(exenv, rou, 7, cuf, NULL))) {
     ;;      ifBLIN_QW0("pgoblin_db0_create");
     ;;      goto out;
     ;;  }
     ;;  rou->cortege = -1;
-    ;;  rou->flags |= PGOBLIN_PQRESULT;
-    MARK_IO_MIFE_WENT(rn[PGO_COUT]);;
+    ;;  rou->parmask |= PGOBLIN_PQRESULT;
+    MARK_IO_MIFE_WENT(rou);;
     for (int i = 0; i < 4; ++i) free(buf[i]);
 out:
-    ifBLIN_QX3("- %"BLIN_D, ex);
+    ifBLIN_QX3("- %d", ex);
+    return(ex);
+#   undef blin_internal_flags
+}
+
+int
+/**********************************************************************
+ **********************************************************************
+ **                                                                  **/
+pgoblin_getspq(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
+ **                                                                  **
+ **********************************************************************
+ **********************************************************************/
+    ifBLIN_QX3( "+getspq......");
+    ifBLIN_QX3("- %d", EX_UNAVAILABLE);
+    return(EX_UNAVAILABLE);
+}
+
+int
+/**********************************************************************
+ **********************************************************************
+ **                                                                  **/
+pgoblin_getstr(pgoblin_exenv *exenv, pgoblin_nr *rn) {              /**
+ **                                                                  **
+ **********************************************************************
+ **********************************************************************/
+    int          ex = EX_OK;
+    char        *cont;
+    pgoblin_rio *rou;
+    pgoblin_rio *rin;
+    FILE        *fi = NULL;
+    ssize_t      bl = 0;
+    ssize_t      i;
+    int          j;
+    int          l;
+
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    ifBLIN_QX3( "+getstr %c.%c...", pgoblin_regn[rn[PGO_COUT]], pgoblin_regn[rn[PGO_CIN]]);
+    GET_RIO(rou, exenv->options, rn[PGO_COUT]);
+    GET_RIO(rin, exenv->options, rn[PGO_CIN]);
+    if  (!(fi = pgoblin_io_funopen(rou))) {
+        ifBLIN_QW0("getstr funopen #0 on %d", rn[PGO_COUT]);
+        ex = EX_OSERR;
+        goto out;
+    }
+    for (j = 0, l = 1; !(rin->mife->flags & MIFE_EOFL);) {
+        bl = pgoblin_io_read( rin
+                            , ((ssize_t)1 << (PGOBLIN_RAWIOBUFL + 16))
+                            , mife_offset(rin->mife) + bl
+                            );
+        if  (0 > bl) {
+            ifBLIN_QW0("pgoblin_io_read");
+            ex = EX_IOERR;
+            break;
+        }
+        if  (!(cont = mife_pointer(rin->mife))) {
+            ifBLIN_QW0("mife_pointer");
+            ex = EX_IOERR;
+            break;
+        }
+        for (i = 0; i < bl; i++, cont++) {
+            switch (*cont) {
+            case '\\':
+                if  (!!l && (0 > fprintf(fi, "%d\t", j))) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                l = 0;
+                if  (0 > fprintf(fi, "\\\\")) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                break;
+            case '\0':
+                if  (!!l && (0 > fprintf(fi, "%d\t", j))) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                l = 0;
+                if  (0 > fprintf(fi, "\\0")) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                break;
+            case '\t':
+                if  (!!l && (0 > fprintf(fi, "%d\t", j))) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                l = 0;
+                if  (0 > fprintf(fi, "\\t")) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                break;
+            case '\n':
+                if  (!!l && (0 > fprintf(fi, "%d\t", j))) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                l = 1;
+                if  (0 > fprintf(fi, "\n")) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                j++;
+                break;
+            default:
+                if  (!!l && (0 > fprintf(fi, "%d\t", j))) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                l = 0;
+                if  (0 > fprintf(fi, "%c", *cont)) {
+                    ifBLIN_QW0("fprintf");
+                    ex = EX_IOERR;
+                    goto out;
+                }
+                break;
+    }   }   }
+    if  (!l && (0 > fprintf(fi, "\n"))) {
+        ifBLIN_QW0("fprintf");
+        ex = EX_IOERR;
+        goto out;
+    }
+    if  ((~exenv->options->flags & PGOBLIN_COPEND_NOGEN) && (0 > fprintf(fi, "\\.\n"))) {
+        ifBLIN_QW0("fprintf fun");
+        ex = EX_IOERR;
+        goto out;
+    }
+out:
+    if  (0 > pgoblin_io_funclose(rou)) {
+        ifBLIN_QW0("io_funclose on %d", rn[PGO_COUT]);
+        ex = EX_IOERR;
+    }
+    ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
