@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2017..2024 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: db.c,v 1.54 2022/03/20 19:11:52 babolo Exp $"
+#ident "@(#) Copyright (C)2017..2024 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: db.c,v 1.73 2024/07/14 00:00:02 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -50,23 +50,20 @@ int
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_shurecon(pgoblin_exenv *exenv, pgoblin_nr ndb) {         /**
+pgoblin_db_shurecon(pgoblin_rdb *rdb) {                             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     int          ex = EX_OK;
 
-    GET_CON(rdb, exenv->options, ndb);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->shurecon) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->shurecon(exenv, ndb);
+        ex = DBASE(rdb->flags)->shurecon(rdb);
     }
-out:
     ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
@@ -76,22 +73,22 @@ int
 /*******************************************************************************************
  *******************************************************************************************
  **                                                                                       **/
-pgoblin_db_query(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, pgoblin_nr nct) { /**
+pgoblin_db_query(pgoblin_exenv *exenv, pgoblin_nr *rn) {                                 /**
  **                                                                                       **
  *******************************************************************************************
  *******************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     /*    */
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->query) {
         errno = ENOSYS;
-        ex = -EX_UNAVAILABLE;
+        ex = EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->query(exenv, ndb, nou, nct);
+        ex = DBASE(rdb->flags)->query(exenv, rn);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -103,21 +100,21 @@ int
 /*********************************************************************************************
  *********************************************************************************************
  **                                                                                         **/
-pgoblin_db_prepare(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, pgoblin_nr nct) { /**
+pgoblin_db_prepare(pgoblin_exenv *exenv, pgoblin_nr *rn) {                                 /**
  **                                                                                         **
  *********************************************************************************************
  *********************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->prepare) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->prepare(exenv, ndb, nou, nct);
+        ex = DBASE(rdb->flags)->prepare(exenv, rn);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -129,26 +126,21 @@ int
 /************************************************************************************************
  ************************************************************************************************
  **                                                                                            **/
-pgoblin_db_execute( pgoblin_exenv *exenv                                                      /**/
-                  , pgoblin_nr     ndb                                                        /**/
-                  , pgoblin_nr     nou                                                        /**/
-                  , pgoblin_nr     nct                                                        /**/
-                  , pgoblin_nr     nin                                                        /**/
-                  ) {                                                                         /**
+pgoblin_db_execute(pgoblin_exenv *exenv, pgoblin_nr *rn) {                                    /**
  **                                                                                            **
  ************************************************************************************************
  ************************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->execute) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->execute(exenv, ndb, nou, nct, nin);
+        ex = DBASE(rdb->flags)->execute(exenv, rn);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -157,19 +149,17 @@ out:
 }
 
 int64_t
-/***********************************************************************
- ***********************************************************************
- **                                                                   **/
-pgoblin_db_resinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind) { /**
- **                                                                   **
- ***********************************************************************
- ***********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio *rio;
-    int64_t      ex = 0;
+/*************************************************************************
+ *************************************************************************
+ **                                                                     **/
+pgoblin_db_resinfo(pgoblin_rio *rio, int kind) {                       /**
+ **                                                                     **
+ *************************************************************************
+ *************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
+    int64_t      ex = -EX_SOFTWARE;
 
-    GET_RIO(rio, exenv->options, nio);
-    ifBLIN_QX3("+ [%d]%d", !rio->pq ? -1 : (*rio->pq & PGOBLIN_DB_TYPE), kind);
+    ifBLIN_QX3("+ [%d]%d", !rio->pq ? -1 : (int)(*rio->pq & PGOBLIN_DB_TYPE), kind);
     if  (!rio->pq) {
         ifBLIN_QX1("No pq");
         errno = EFAULT;
@@ -178,124 +168,112 @@ pgoblin_db_resinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind) { /**
         ifBLIN_QX1("No DB or resinfo");
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
-    } else if (  !!(rio->flags & PGOBLIN_PURGE)
+    } else if (  !!(rio->purge)
               && ((PGOBLIN_Ntuples == kind) || (PGOBLIN_Nfields == kind))
               ) {
         ex = 0;
-    } else if (  (1 < (ex = DBASE(*rio->pq)->resinfo(exenv, nio, kind)))
+    } else if (  (1 < (ex = DBASE(*rio->pq)->resinfo(rio, kind)))
               && (PGOBLIN_Ntuples == kind)
               && (0 <= rio->cortege)
               ) {
         ex = 1;
     }
-    ifBLIN_QX3("- %d", ex);
-out:
+    ifBLIN_QX3("- %"BLIN_O"d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
 
 const char *
-/************************************************************************************
- ************************************************************************************
- **                                                                                **/
-pgoblin_db_subinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int64_t sub) { /**
- **                                                                                **
- ************************************************************************************
- ************************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/**************************************************************************************
+ **************************************************************************************
+ **                                                                                  **/
+pgoblin_db_subinfo(pgoblin_rio *rio, int kind, int64_t sub) {                       /**
+ **                                                                                  **
+ **************************************************************************************
+ **************************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     const char   *c = NULL;
 
-    GET_RIO(rio, exenv->options, nio);
-    ifBLIN_QX3("+ [%d]%d %d", !rio->pq ? -1 : (*rio->pq & PGOBLIN_DB_TYPE), kind, sub);
+    ifBLIN_QX3("+ [%d]%d %"BLIN_O"d", !rio->pq ? -1 : (int)(*rio->pq & PGOBLIN_DB_TYPE), kind, sub);
     if  (!rio->pq) {
         errno = EFAULT;
     } else if (!DBASE(*rio->pq) || !DBASE(*rio->pq)->subinfo) {
         errno = ENOSYS;
-    } else if (  !!(rio->flags & PGOBLIN_PURGE)
+    } else if (  !!(rio->purge)
               && ((PGOBLIN_TypeNm == kind) || (PGOBLIN_ColName == kind))
               ) {
         errno = ENOENT;
     } else {
-        c = DBASE(*rio->pq)->subinfo(exenv, nio, kind, sub);
+        c = DBASE(*rio->pq)->subinfo(rio, kind, sub);
     }
-out:
     ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
 }
 
 char *
-/**********************************************************************
- **********************************************************************
- **                                                                  **/
-pgoblin_db_erinfo(pgoblin_exenv *exenv, pgoblin_nr nio) {           /**
- **                                                                  **
- **********************************************************************
- **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/************************************************************************
+ ************************************************************************
+ **                                                                    **/
+pgoblin_db_erinfo(pgoblin_rio *rio) {                                 /**
+ **                                                                    **
+ ************************************************************************
+ ************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     char         *c = NULL;
 
-    GET_RIO(rio, exenv->options, nio);
-    ifBLIN_QX3("+ [%d]", !rio->pq ? -1 : (*rio->pq & PGOBLIN_DB_TYPE));
+    ifBLIN_QX3("+ [%d]", !rio->pq ? -1 : (int)(*rio->pq & PGOBLIN_DB_TYPE));
     if  (!rio->pq) {
         errno = EFAULT;
     } else if (!DBASE(*rio->pq) || !DBASE(*rio->pq)->erinfo) {
         errno = ENOSYS;
     } else {
-        c = DBASE(*rio->pq)->erinfo(exenv, nio);
+        c = DBASE(*rio->pq)->erinfo(rio);
     }
-out:
     ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
 }
 
 ssize_t
-/*****************************************************************************************
- *****************************************************************************************
- **                                                                                     **/
-pgoblin_db_valinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int row, int col) { /**
- **                                                                                     **
- *****************************************************************************************
- *****************************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/*******************************************************************************************
+ *******************************************************************************************
+ **                                                                                       **/
+pgoblin_db_valinfo(pgoblin_rio *rio, int kind, ssize_t row, ssize_t col) {               /**
+ **                                                                                       **
+ *******************************************************************************************
+ *******************************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     ssize_t       ex = PGOBLIN_SSIZE_MIN;
 
-    GET_RIO(rio, exenv->options, nio);
-    ifBLIN_QX3("+ [%d]%d %d %d", !rio->pq ? -1 : (*rio->pq & PGOBLIN_DB_TYPE), kind, row, col);
+    ifBLIN_QX3("+ [%d]%d %zd %zd", !rio->pq ? -1 : (int)(*rio->pq & PGOBLIN_DB_TYPE), kind, row, col);
     if  (!rio->pq) {
         errno = EFAULT;
     } else if (!DBASE(*rio->pq) || !DBASE(*rio->pq)->valinfo) {
         errno = ENOSYS;
-    } else if (!!(rio->flags & PGOBLIN_PURGE) && (PGOBLIN_Length == kind)) {
+    } else if (!!rio->purge && (PGOBLIN_Length == kind)) {
         ex = 0;
     } else {
-        ex = DBASE(*rio->pq)->valinfo(exenv, nio, kind, (0 > rio->cortege) ? row : rio->cortege, col);
+        ex = DBASE(*rio->pq)->valinfo(rio, kind, (0 > rio->cortege) ? row : rio->cortege, col);
     }
-out:
-    ifBLIN_QX3("- %d", ex);
+    ifBLIN_QX3("- %zd", ex);
     return(ex);
 #   undef blin_internal_flags
 }
 
 char *
-/********************************************************************************
- ********************************************************************************
- **                                                                            **/
-pgoblin_db_getvalue(pgoblin_exenv *exenv, pgoblin_nr nio, int row, int col) { /**
- **                                                                            **
- ********************************************************************************
- ********************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio *rio;
+/**********************************************************************************
+ **********************************************************************************
+ **                                                                              **/
+pgoblin_db_getvalue(pgoblin_rio *rio, ssize_t row, ssize_t col) {               /**
+ **                                                                              **
+ **********************************************************************************
+ **********************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     char        *c = NULL;
 
-    GET_RIO(rio, exenv->options, nio);
-    ifBLIN_QX3( "+ [%d]%d %d(%d)"
-              , (!rio->pq) ? -1 : (*(rio->pq) & PGOBLIN_DB_TYPE)
+    ifBLIN_QX3( "+ [%d]%zd %zd(%zd)"
+              , (!rio->pq) ? -1 : (int)(*(rio->pq) & PGOBLIN_DB_TYPE)
               , row
               , col
               , rio->cortege
@@ -304,12 +282,11 @@ pgoblin_db_getvalue(pgoblin_exenv *exenv, pgoblin_nr nio, int row, int col) { /*
         errno = EFAULT;
     } else if (!DBASE(*(rio->pq)) || !DBASE(*(rio->pq))->getvalue) {
         errno = ENOSYS;
-    } else if (!!(rio->flags & PGOBLIN_PURGE)) {
+    } else if (!!rio->purge) {
         errno = ENOENT;
     } else {
-        c = DBASE(*(rio->pq))->getvalue(exenv, nio, (0 > rio->cortege) ? row : rio->cortege, col);
+        c = DBASE(*(rio->pq))->getvalue(rio, (0 > rio->cortege) ? row : rio->cortege, col);
     }
-out:
     ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
@@ -319,21 +296,21 @@ int
 /**********************************************************************************************
  **********************************************************************************************
  **                                                                                          **/
-pgoblin_db_getstream(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, int quant) {     /**
+pgoblin_db_getstream(pgoblin_exenv *exenv, pgoblin_nr *rn, int quant) {                     /**
  **                                                                                          **
  **********************************************************************************************
  **********************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]%d", rdb->flags & PGOBLIN_DB_TYPE, quant);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->getstream) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->getstream(exenv, ndb, nou, quant);
+        ex = DBASE(rdb->flags)->getstream(exenv, rn, quant);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -345,21 +322,21 @@ int
 /**************************************************************************
  **************************************************************************
  **                                                                      **/
-pgoblin_db_getcopy(pgoblin_exenv *exenv, pgoblin_nr ndb, void **buf) {  /**
+pgoblin_db_getcopy(pgoblin_exenv *exenv, pgoblin_nr *rn, void **buf) {  /**
  **                                                                      **
  **************************************************************************
  **************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->getcopy) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->getcopy(exenv, ndb, buf);
+        ex = DBASE(rdb->flags)->getcopy(exenv, rn, buf);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -368,24 +345,24 @@ out:
 }
 
 int
-/******************************************************************************************
- ******************************************************************************************
- **                                                                                      **/
-pgoblin_db_putcopy(pgoblin_exenv *exenv, pgoblin_nr ndb, const void *buf, int len) {    /**
- **                                                                                      **
- ******************************************************************************************
- ******************************************************************************************/
+/*******************************************************************************************
+ *******************************************************************************************
+ **                                                                                       **/
+pgoblin_db_putcopy(pgoblin_exenv *exenv, pgoblin_nr *rn, const void *buf, ssize_t len) { /**
+ **                                                                                       **
+ *******************************************************************************************
+ *******************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
-    ifBLIN_QX3("+ [%d]%d", rdb->flags & PGOBLIN_DB_TYPE, len);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
+    ifBLIN_QX3("+ [%d]%zd", rdb->flags & PGOBLIN_DB_TYPE, len);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->putcopy) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->putcopy(exenv, ndb, buf, len);
+        ex = DBASE(rdb->flags)->putcopy(exenv, rn, buf, len);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -397,21 +374,21 @@ int
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_endstream(pgoblin_exenv *exenv, pgoblin_nr ndb) {        /**
+pgoblin_db_endstream(pgoblin_exenv *exenv, pgoblin_nr *rn) {        /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->endstream) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->endstream(exenv, ndb);
+        ex = DBASE(rdb->flags)->endstream(exenv, rn);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -423,23 +400,20 @@ int
 /************************************************************************
  ************************************************************************
  **                                                                    **/
-pgoblin_db_tarry(pgoblin_exenv *exenv, pgoblin_nr ndb, int timeout) { /**
+pgoblin_db_tarry(pgoblin_rdb *rdb, int timeout) {                     /**
  **                                                                    **
  ************************************************************************
  ************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     int          ex = 0;
 
-    GET_CON(rdb, exenv->options, ndb);
     ifBLIN_QX3("+ [%d]%d", rdb->flags & PGOBLIN_DB_TYPE, timeout);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->tarry) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->tarry(exenv, ndb, timeout);
+        ex = DBASE(rdb->flags)->tarry(rdb, timeout);
     }
-out:
     ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
@@ -449,18 +423,15 @@ void *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_notifies(pgoblin_exenv *exenv, pgoblin_nr ndb) {         /**
+pgoblin_db_notifies(pgoblin_rdb *rdb) {                             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     char        *c = NULL;
 
-    GET_CON(rdb, exenv->options, ndb);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
-    c = DBASE(rdb->flags)->notifies(exenv, ndb);
-out:
+    c = DBASE(rdb->flags)->notifies(rdb);
     ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
@@ -470,22 +441,21 @@ int
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_finish(pgoblin_exenv *exenv, pgoblin_nr ndb) {           /**
+pgoblin_db_finish(pgoblin_rdb *rdb) {                               /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     int          ex = 0;
 
-    GET_CON(rdb, exenv->options, ndb);
+    ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags)->finish) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->finish(exenv, ndb);
+        ex = DBASE(rdb->flags)->finish(rdb);
     }
-out:
+    ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
@@ -494,23 +464,22 @@ int
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_clear(pgoblin_exenv *exenv, pgoblin_nr nio) {            /**
+pgoblin_db_clear(pgoblin_rio *rio) {                                /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio *rio;
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     int          ex = 0;
 
-    GET_RIO(rio, exenv->options, nio);
+    ifBLIN_QX3( "+ [%d]", (!rio || !rio->pq) ? -1 : (int)(*(rio->pq) & PGOBLIN_DB_TYPE));
     if  (!!rio && !!rio->pq) {
         if  (!DBASE(*rio->pq) || !DBASE(*rio->pq)->clear) {
             errno = ENOSYS;
             ex = -EX_UNAVAILABLE;
         } else {
-            ex = DBASE(*rio->pq)->clear(exenv, nio);
+            ex = DBASE(*rio->pq)->clear(rio);
     }   }
-out:
+    ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
@@ -519,21 +488,20 @@ char *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db_erconn(pgoblin_exenv *exenv, pgoblin_nr ndb) {           /**
+pgoblin_db_erconn(pgoblin_rdb *rdb) {                               /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     char        *c = NULL;
 
-    GET_CON(rdb, exenv->options, ndb);
+    ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags)->erconn) {
         errno = ENOSYS;
     } else {
-        c = DBASE(rdb->flags)->erconn(exenv, ndb);
+        c = DBASE(rdb->flags)->erconn(rdb);
     }
-out:
+    ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
 }
@@ -542,26 +510,21 @@ int
 /************************************************************************************************
  ************************************************************************************************
  **                                                                                            **/
-pgoblin_db_prexec( pgoblin_exenv *exenv                                                       /**/
-                 , pgoblin_nr     ndb                                                         /**/
-                 , pgoblin_nr     nou                                                         /**/
-                 , pgoblin_nr     nct                                                         /**/
-                 , pgoblin_nr     nin                                                         /**/
-                 ) {                                                                          /**
+pgoblin_db_prexec(pgoblin_exenv *exenv, pgoblin_nr *rn) {                                     /**
  **                                                                                            **
  ************************************************************************************************
  ************************************************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
-    int          ex = 0;
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
     ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags) || !DBASE(rdb->flags)->execute) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->prexec(exenv, ndb, nou, nct, nin);
+        ex = DBASE(rdb->flags)->prexec(exenv, rn);
     }
 out:
     ifBLIN_QX3("- %d", ex);
@@ -573,22 +536,21 @@ int
 /*************************************************************************
  *************************************************************************
  **                                                                     **/
-pgoblin_db_denote(pgoblin_exenv *exenv, pgoblin_nr ndb, void **note) { /**
+pgoblin_db_denote(pgoblin_rdb *rdb, void **note) {                     /**
  **                                                                     **
  *************************************************************************
  *************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
-    int          ex = 0;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
+    int          ex = -EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
+    ifBLIN_QX3("+ [%d]", rdb->flags & PGOBLIN_DB_TYPE);
     if  (!DBASE(rdb->flags)->denote) {
         errno = ENOSYS;
         ex = -EX_UNAVAILABLE;
     } else {
-        ex = DBASE(rdb->flags)->denote(exenv, ndb, note);
+        ex = DBASE(rdb->flags)->denote(rdb, note);
     }
-out:
+    ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
