@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2008..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2008..2024 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,15 +24,15 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2008..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: db_0.c,v 1.90 2022/01/08 22:59:07 babolo Exp $"
+#ident "@(#) Copyright (C)2008..2024 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: db_0.c,v 1.110 2024/07/14 00:00:02 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
 #define PGOBLIN_MODULE   2
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -63,30 +63,36 @@ struct pgoblin_realpq {                      /*  Pq
 #   define    PGOBLIN_DB_0FREET 0x080000   /*   serv  free()                          */
 #   define    PGOBLIN_DB_0FREEL 0x100000   /* llen  free()                                       */
 #   define    PGOBLIN_DB_0INDIR 0xC00000   /*   Pq    Pq (1),  (2), (3)  */
-    int       ncol;                        /*                               */
+#if BLINAP > 4
+    int32_t   dummy;
+#endif
+    size_t    nrow;                        /*                                 */
+    size_t    ncol;                        /*                               */
     char    **cell;                        /*                */
     size_t   *llen;                        /*        *
                                             *  ,         *
                                             *  strlen()                                  */
     char    **serv;                        /*  ͣ                                */
-    int       nrow;                        /*                                 */
 };
 
 pgoblin_realpq *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_db00(BLIN_flag flags) {                                     /**
+pgoblin_db00(pgoblin_exenv *exenv) {                                /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-    pgoblin_realpq *rsv = NULL;
+    int             loaded;
+    pgoblin_realpq *rsv   = NULL;
 
-#   define blin_internal_flags (flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     if  (!(rsv = calloc(1, sizeof(pgoblin_realpq)))) {
         ifBLIN_QW0("");
+    } else if (0 > (loaded = pgoblin_load(exenv, 3, "0", -1))) {
+        ifBLIN_QW0("");
     } else {
-        rsv->flags = (flags & BLIN_MASK) | pgoblin_load(pgoblin_options, 3, "0", -1);
+        rsv->flags = (exenv->options->flags & BLIN_MASK) | (u_int)loaded;
     }
     return(rsv);
 #   undef blin_internal_flags
@@ -95,31 +101,29 @@ pgoblin_db00(BLIN_flag flags) {                                     /**
 static pgoblin_realpq *
 /**********************************************************************
  **                                                                  **/
-resv(pgoblin_rdb *rdb) {                                            /**
+resv(pgoblin_exenv *exenv) {                                        /**
  **                                                                  **
  **********************************************************************/
-    return(pgoblin_db00(rdb->flags));
+    return(pgoblin_db00(exenv));
 }
 
 int
-/**********************************************************************
- **                                                                  **/
-pgoblin_db0_create( pgoblin_exenv      *exenv                       /**/
-/**/              , pgoblin_nr          nou                         /**/
-/**/              , int                 cnt                         /**/
-/**/              , const char * const *val                         /**/
-/**/              , const int          *len                         /**/
-/**/              ) {                                               /**
- **                                                                  **
- **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio   *rou;
+/*******************************************************************************************
+ *******************************************************************************************
+ **                                                                                       **/
+pgoblin_db0_create( pgoblin_exenv      *exenv                                            /**/
+                  , pgoblin_rio        *rou                                              /**/
+                  , size_t              cnt                                              /**/
+                  , const char * const *val                                              /**/
+                  , const size_t       *len                                              /**/
+                  ) {                                                                    /**
+ **                                                                                       **
+ *******************************************************************************************
+ *******************************************************************************************/
+#   define blin_internal_flags (rou->flags & BLIN_MASK)
     int            ex = EX_OK;
 
-    GET_RIO(rou, exenv->options, nou);
-    rou->pq = pgoblin_db00( (exenv->options->flags & ~(BLIN_MASK | PGOBLIN_DB_TYPE))
-                          | (pgoblin_db_0.flags & (BLIN_MASK | PGOBLIN_DB_TYPE))
-                          );
+    rou->pq = pgoblin_db00(exenv);
     if  (!rou->pq) {
         ifBLIN_QX0("No mem #1");
         errno = ENOMEM;
@@ -139,11 +143,11 @@ pgoblin_db0_create( pgoblin_exenv      *exenv                       /**/
         rou->pq->flags |= PGOBLIN_DB_0FREEL | PGOBLIN_DB_0FREEC | PGOBLIN_DB_0FREEA;
         rou->pq->nrow = 1;
         rou->pq->ncol = cnt;
-        for (int i = 0; i < cnt; i++) {
+        for (size_t i = 0; i < cnt; i++) {
             if  (!val || !val[i]) {
                 rou->pq->cell[i] = NULL;
                 rou->pq->llen[i] = 0;
-            } else if (!len || (len[i] < 0)) {
+            } else if (!len) {
                 rou->pq->cell[i] = strdup(val[i]);
                 rou->pq->llen[i] = strlen(val[i]);
             } else {
@@ -152,7 +156,6 @@ pgoblin_db0_create( pgoblin_exenv      *exenv                       /**/
                 rou->pq->cell[i][len[i]] = '\0';
                 rou->pq->llen[i] = len[i];
     }   }   }
-out:
     if  (!!ex) rou->pq = NULL;
     return(ex);
 #   undef blin_internal_flags
@@ -161,11 +164,11 @@ out:
 static int
 /**********************************************************************
  **                                                                  **/
-init(pgoblin_main *options, int slot) {                             /**
+init(pgoblin_exenv *exenv, u_int slot) {                            /**
  **                                                                  **
  **********************************************************************/
     pgoblin_db_0.flags = (pgoblin_db_0.flags & ~(BLIN_MASK | PGOBLIN_DB_TYPE))
-                       | (options->flags & BLIN_MASK)
+                       | (exenv->options->flags & BLIN_MASK)
                        | (slot & PGOBLIN_DB_TYPE)
     ;
     return(0);
@@ -174,30 +177,28 @@ init(pgoblin_main *options, int slot) {                             /**
 static int
 /**********************************************************************
  **                                                                  **/
-fini(pgoblin_main *options) {                                       /**
+fini(pgoblin_exenv *exenv) {                                        /**
  **                                                                  **
  **********************************************************************/
+    (void)exenv;
     return(0);
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-shurecon(pgoblin_exenv *exenv, pgoblin_nr ndb) {                    /**
+shurecon(pgoblin_rdb *rdb) {                                        /**
  **                                                                  **
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     int          ex = EX_OK;
 
-    GET_CON(rdb, exenv->options, ndb);
-    MARK_R_CONN_GO(ndb);;
+    MARK_R_CONN_GO(rdb);;
     ;;  rdb->conn = NULL;                                                              /* Pc  */
     ;;  rdb->flags = (rdb->flags & ~(BLIN_MASK | PGOBLIN_DB_TYPE))
  /* ;; */          | (pgoblin_db_0.flags & (BLIN_MASK | PGOBLIN_DB_TYPE))
     ;;  ;
-    MARK_R_CONN_WENT(ndb);;
-out:
+    MARK_R_CONN_WENT(rdb);;
     ifBLIN_QX4("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
@@ -206,7 +207,7 @@ out:
 static int
 /********************************************************************************
  **                                                                            **/
-query(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, pgoblin_nr nct) { /**
+query(pgoblin_exenv *exenv, pgoblin_nr *rn) {                                 /**
  **                                                                            **
  ********************************************************************************
  **  #select      NULL,                       **
@@ -216,19 +217,20 @@ query(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, pgoblin_nr nct) { /*
     pgoblin_rdb   *rdb;
     pgoblin_rio   *rou;
     pgoblin_rio   *rct;
-    int            ex = EX_OK;
+    int            ex = EX_SOFTWARE;
     char          *q;
 
-    GET_CON(rdb, exenv->options, ndb);
-    GET_RIO(rou, exenv->options, nou);
-    GET_RIO(rct, exenv->options, nct);
-    if  (!(rou->pq = resv(rdb))) {
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
+    GET_RIO(rou, exenv->options, rn[PGO_COUT]);
+    GET_RIO(rct, exenv->options, rn[PGO_CCTL]);
+    if  (!(rou->pq = resv(exenv))) {
         ifBLIN_QX0("No mem");
         errno = ENOMEM;
         ex = EX_OSERR;
     } else if (rct->text) {
         rou->pq->nrow = strtoul(rct->text, &q, 0);
         rou->pq->ncol = strtoul(q, &q, 0);
+        ex = EX_OK;
     }
 out:
     ifBLIN_QX4("- %d", ex);
@@ -237,19 +239,19 @@ out:
 }
 
 static int
-/*****************************************************************************************************
- **                                                                                                 **/
-execute(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, pgoblin_nr nct, pgoblin_nr nin) {    /**
- **                                                                                                 **
- *****************************************************************************************************/
+/*****************************************************
+ **                                                 **/
+execute(pgoblin_exenv *exenv, pgoblin_nr *rn) {    /**
+ **                                                 **
+ *****************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     pgoblin_rdb *rdb;
     pgoblin_rio *rou;
     int          ex = EX_SOFTWARE;
 
-    GET_CON(rdb, exenv->options, ndb);
-    GET_RIO(rou, exenv->options, nou);
-    if  (!(rou->pq = resv(rdb))) {
+    GET_CON(rdb, exenv->options, rn[PGO_CCON]);
+    GET_RIO(rou, exenv->options, rn[PGO_COUT]);
+    if  (!(rou->pq = resv(exenv))) {
         ifBLIN_QX0("No mem #1");
         errno = ENOMEM;
         ex = EX_OSERR;
@@ -262,55 +264,69 @@ out:
 static int64_t
 /**********************************************************************
  **                                                                  **/
-resinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind) {           /**
+resinfo(pgoblin_rio *rio, int kind) {                               /**
  **                                                                  **
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     int64_t       ex = 0;
 
-    GET_RIO(rio, exenv->options, nio);
+    ifBLIN_QX3("+ %d", kind);
     if  (!rio->pq) {
         errno = EFAULT;
         ex = -1;
     } else {
         switch (kind) {
         case PGOBLIN_Ntuples:
-            ex = rio->pq->nrow;
+            ex = (int64_t)rio->pq->nrow;
             break;
         case PGOBLIN_Nfields:
-            ex = rio->pq->ncol;
+            ex = (int64_t)rio->pq->ncol;
             break;
         default:
             ex = -1;
             errno = ENOTSUP;
     }   }
-out:
+    ifBLIN_QX3("- %"BLIN_O"d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
 
 static const char *
-/**************************************************************************
- **                                                                      **/
-subinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int64_t sub) {  /**
- **                                                                      **
- **************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/***************************************************************************
+ **                                                                       **/
+subinfo(pgoblin_rio *rio, int kind, int64_t sub) {                       /**
+ **                                                                       **
+ ***************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     const char   *cx = NULL;
+    size_t        s  = (size_t)sub;
 
-    GET_RIO(rio, exenv->options, nio);
+    ifBLIN_QX3("+ %d sub=%"BLIN_O"d", kind, sub);
     switch (kind) {
     case PGOBLIN_ColName:
-        if  (!!rio->pq->serv && (rio->pq->ncol > sub)) cx = rio->pq->serv[sub];
+        if  (0 > sub) {
+            ifBLIN_QX0("sub=%"BLIN_O"d", sub);
+            errno = EDOM;
+            goto out;
+        }
+        if  (!!rio->pq->serv && (rio->pq->ncol > s)) cx = rio->pq->serv[s];
         break;
     case PGOBLIN_TypeNm:
-        if  (!!rio->pq->serv && (rio->pq->ncol > sub)) cx = rio->pq->serv[sub + rio->pq->ncol];
+        if  (0 > sub) {
+            ifBLIN_QX0("sub=%"BLIN_O"d", sub);
+            errno = EDOM;
+            goto out;
+        }
+        if  (!!rio->pq->serv && (rio->pq->ncol > s)) cx = rio->pq->serv[s + rio->pq->ncol];
         break;
     default:;
     }
 out:
+    if  (!cx) {
+        ifBLIN_QX3("-");
+    } else {
+        ifBLIN_QX3("- %s", cx);
+    }
     return(cx);
 #   undef blin_internal_flags
 }
@@ -318,39 +334,47 @@ out:
 static char *
 /**********************************************************************
  **                                                                  **/
-erinfo(pgoblin_exenv *exenv, pgoblin_nr nio) {                      /**
+erinfo(pgoblin_rio *rio) {                                          /**
  **                                                                  **
  **********************************************************************/
     char *cx = NULL;
 
+    (void)rio;
     return(cx);
 }
 
 static ssize_t
-/******************************************************************************
- **                                                                          **/
-valinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int row, int col) { /**
- **                                                                          **
- ******************************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/********************************************************************************
+ **                                                                            **/
+valinfo(pgoblin_rio *rio, int kind, ssize_t row, ssize_t col) {               /**
+ **                                                                            **
+ ********************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     ssize_t       ex = PGOBLIN_SSIZE_MIN;
 
-    GET_RIO(rio, exenv->options, nio);
+    ifBLIN_QX3("+ %zd %zd", row, col);
+    if  (!rio) {
+        errno = EFAULT;
+        goto out;
+    }
     switch (kind) {
     case PGOBLIN_IsNull:
         if  (rio->pq) {
             if  (!rio->pq->cell) {
                 ex = 1;
-            } else if ((row >= 0) && (row < rio->pq->nrow) && (col >= 0) && (col < rio->pq->ncol)) {
-                ex = !rio->pq->cell[INDX(row, col)];
+            } else if (  (row >= 0)
+                      && ((size_t)row < rio->pq->nrow)
+                      && (col >= 0)
+                      && ((size_t)col < rio->pq->ncol)
+                      ) {
+                ex = !rio->pq->cell[INDX((size_t)row, (size_t)col)];
             } else {
-                ifBLIN_QX1( "+ row %d %s in [0..%d], col %d %s in [0..%d]"
+                ifBLIN_QX1( "+ row %zd %s in [0..%zd], col %zd %s in [0..%zd]"
                           , row
-                          , (row >= 0 && row < rio->pq->nrow) ? "" : "NOT"
+                          , (row >= 0 && (size_t)row < rio->pq->nrow) ? "" : "NOT"
                           , rio->pq->nrow - 1
                           , col
-                          , (col >= 0 && col < rio->pq->ncol) ? "" : "NOT"
+                          , (col >= 0 && (size_t)col < rio->pq->ncol) ? "" : "NOT"
                           , rio->pq->ncol - 1
                           );
                 errno = EINVAL;
@@ -358,43 +382,52 @@ valinfo(pgoblin_exenv *exenv, pgoblin_nr nio, int kind, int row, int col) { /**
         } else if (row == 0 && col == 0) {
             ex = 1;
         } else {
-            ifBLIN_QX0( "row %d col %d in emty", row, col);
+            ifBLIN_QX0( "row %zd col %zd in emty", row, col);
             errno = EINVAL;
         }
         break;
     case PGOBLIN_Length:
-        if  (0 > (ex = valinfo(exenv, nio, PGOBLIN_IsNull, row, col))) {
-        } else if (!!ex) {
-            ex = 0;
+        if  (!rio->pq) {
+            errno = EFAULT;
+        } else if (  (0 > row)
+                  || ((size_t)row >= rio->pq->nrow)
+                  || (0 > col)
+                  || ((size_t)col >= rio->pq->ncol)
+                  ) {
+            errno = EINVAL;
+        } else if (!!valinfo(rio, PGOBLIN_IsNull, row, col)) {
+            errno = EFAULT;
         } else if (rio->pq->llen) {
-            ex = rio->pq->llen[INDX(row, col)];
+            ex = (ssize_t)rio->pq->llen[INDX((size_t)row, (size_t)col)];
         } else if (rio->pq->cell) {
-            ex = strlen(rio->pq->cell[INDX(row, col)]);
+            ex = (ssize_t)strlen(rio->pq->cell[INDX((size_t)row, (size_t)col)]);
         }
         break;
     default:
         errno = ENOTSUP;
     }
 out:
+    ifBLIN_QX3("- %zd", ex);
     return(ex);
 #   undef blin_internal_flags
 }
 
 static char *
-/**********************************************************************
- **                                                                  **/
-getvalue(pgoblin_exenv *exenv, pgoblin_nr nio, int row, int col) {  /**
- **                                                                  **
- **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+/************************************************************************
+ **                                                                    **/
+getvalue(pgoblin_rio *rio, ssize_t row, ssize_t col) {                /**
+ **                                                                    **
+ ************************************************************************/
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     char         *c = NULL;
 
-    GET_RIO(rio, exenv->options, nio);
-    if  (!!rio->pq && !valinfo(exenv, nio, PGOBLIN_IsNull, row, col)) {
-        c = rio->pq->cell[INDX(row, col)];
+    ifBLIN_QX3("+ %zd %zd", row, col);
+    if  (!rio || !rio->pq) {
+        errno = EINVAL;
+    } else if (!valinfo(rio, PGOBLIN_IsNull, row, col)) {
+        c = rio->pq->cell[INDX((size_t)row, (size_t)col)];
     }
-out:
+    ifBLIN_QX3("- %s", c);
     return(c);
 #   undef blin_internal_flags
 }
@@ -402,78 +435,96 @@ out:
 static int
 /***********************************************************************************
  **                                                                               **/
-getstream(pgoblin_exenv *exenv, pgoblin_nr ndb, pgoblin_nr nou, int quant) {     /**
+getstream(pgoblin_exenv *exenv, pgoblin_nr *rn, int quant) {                     /**
  **                                                                               **
  ***********************************************************************************/
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    (void)rn;
+    (void)quant;
     ifBLIN_QX0("Unimplemented in 0");
     errno = ENOTSUP;
     return(-1);
+#   undef blin_internal_flags
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-getcopy(pgoblin_exenv *exenv, pgoblin_nr ndb, void **buf) {         /**
+getcopy(pgoblin_exenv *exenv, pgoblin_nr *rn, void **buf) {         /**
  **                                                                  **
  **********************************************************************/
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    (void)rn;
+    (void)buf;
     ifBLIN_QX0("Unimplemented in 0");
     errno = ENOTSUP;
     return(-1);
+#   undef blin_internal_flags
 }
 
 static int
-/****************************************************************************
- **                                                                        **/
-putcopy(pgoblin_exenv *exenv, pgoblin_nr ndb, const void *buf, int nby) { /**
- **                                                                        **
- ****************************************************************************/
+/********************************************************************************
+ **                                                                            **/
+putcopy(pgoblin_exenv *exenv, pgoblin_nr *rn, const void *buf, ssize_t len) { /**
+ **                                                                            **
+ ********************************************************************************/
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    (void)rn;
+    (void)buf;
+    (void)len;
     ifBLIN_QX0("Unimplemented in 0");
     errno = ENOTSUP;
     return(-1);
+#   undef blin_internal_flags
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-endstream(pgoblin_exenv *exenv, pgoblin_nr ndb) {                   /**
+endstream(pgoblin_exenv *exenv, pgoblin_nr *rn) {                   /**
  **                                                                  **
  **********************************************************************/
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    (void)rn;
     ifBLIN_QX0("Unimplemented in 0");
     errno = ENOTSUP;
     return(-1);
+#   undef blin_internal_flags
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-tarry(pgoblin_exenv *exenv, pgoblin_nr ndb, int timeout) {          /**
+tarry(pgoblin_rdb *rdb, int timeout) {                              /**
  **                                                                  **
  **********************************************************************/
+    (void)rdb;
+    (void)timeout;
     return(0);
 }
 
 static void*
 /**********************************************************************
  **                                                                  **/
-notifies(pgoblin_exenv *exenv, pgoblin_nr ndb) {                    /**
+notifies(pgoblin_rdb *rdb) {                                        /**
  **                                                                  **
  **********************************************************************/
+    (void)rdb;
     return(NULL);
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-finish(pgoblin_exenv *exenv, pgoblin_nr ndb) {                      /**
+finish(pgoblin_rdb *rdb) {                                          /**
  **                                                                  **
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rdb *rdb;
+#   define blin_internal_flags (rdb->flags & BLIN_MASK)
     int          ex = EX_OK;
 
-    GET_CON(rdb, exenv->options, ndb);
+    ifBLIN_QX3("+");
     rdb->conn = NULL;
-out:
+    ifBLIN_QX3("- %d", ex);
     return(ex);
 #   undef blin_internal_flags
 }
@@ -481,15 +532,13 @@ out:
 static int
 /**********************************************************************
  **                                                                  **/
-clear(pgoblin_exenv *exenv, pgoblin_nr nio) {                       /**
+clear(pgoblin_rio *rio) {                                           /**
  **                                                                  **
  **********************************************************************/
-#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    pgoblin_rio  *rio;
+#   define blin_internal_flags (rio->flags & BLIN_MASK)
     int           ex = EX_OK;
-    int           i;
+    size_t        i;
 
-    GET_RIO(rio, exenv->options, nio);
     if  (!!rio->pq) {
         ifBLIN_QX4("No rio->pq");
         if  ((rio->pq->llen) && (rio->pq->flags & PGOBLIN_DB_0FREEL)) {
@@ -519,7 +568,6 @@ clear(pgoblin_exenv *exenv, pgoblin_nr nio) {                       /**
         free(rio->pq);
         rio->pq = NULL;
     }
-out:
     return(ex);
 #   undef blin_internal_flags
 }
@@ -527,9 +575,11 @@ out:
 static int
 /**********************************************************************
  **                                                                  **/
-denote(pgoblin_exenv *exenv, pgoblin_nr ndb, void **note) {         /**
+denote(pgoblin_rdb *rdb, void **note) {                             /**
  **                                                                  **
  **********************************************************************/
+    (void)rdb;
+    (void)note;
     return(EX_OK);
 }
 
@@ -541,25 +591,25 @@ pgoblin_dbases pgoblin_db_0 =
 , init
 , fini
 , shurecon
-, query
-, NULL      /* prepare */
-, execute
+, NULL      /* erconn  */
+, notifies
+, finish
+, tarry
+, denote
+, erinfo
+, clear
 , resinfo
 , subinfo
-, erinfo
 , valinfo
 , getvalue
+, query
+, NULL      /* prepare */
+, NULL      /* prexec  */
+, execute
 , getstream
 , getcopy
 , putcopy
 , endstream
-, tarry
-, notifies
-, finish
-, clear
-, NULL      /* erconn  */
-, NULL      /* prexec  */
-, denote
 , NULL      /* dummy1  */
 , NULL      /* dummy2  */
 };
