@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2007..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2007..2025 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2007..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: descript.c,v 1.98 2022/03/20 19:11:52 babolo Exp $"
+#ident "@(#) Copyright (C)2007..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: descript.c,v 1.123 2025/01/25 00:12:13 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -71,68 +71,29 @@
 #define Ij PGOBLIN_JOB
 #define Is PGOBLIN_STYLE
 
-static pgoblin_dbases *dbasetable[PGOBLIN_DBTABLESZ] =
+static pgoblin_ios *outable[PGOBLIN_OUTABLESZ] =
 {NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
+};
+static pgoblin_ios *iotable[PGOBLIN_IOTABLESZ] =
+{NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
+};
+static pgoblin_ios *intable[PGOBLIN_INTABLESZ] =
+{NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
+};
+static pgoblin_dbases *dbasetable[PGOBLIN_DBTABLESZ] =
+{NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 };
 static pgoblin_styles *styletable[PGOBLIN_STYLETBSZ] =
 {NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 };
 static pgoblin_jobs *jobtable[PGOBLIN_JOBTABSIZ] =
 {NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 ,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
-,NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL, NULL,NULL,NULL,NULL
 };
 
 const char * const pgoblin_syntaxa_flags[] =
@@ -143,25 +104,87 @@ const char * const pgoblin_syntaxa_flags[] =
 };
 
 const pgoblin_syntax pgoblin =
-{ PGOBLIN_DBTABLESZ                                            // dbtablesz
+{ PGOBLIN_OUTABLESZ                                            // outablesz
+, PGOBLIN_IOTABLESZ                                            // iotablesz
+, PGOBLIN_INTABLESZ                                            // intablesz
+, PGOBLIN_DBTABLESZ                                            // dbtablesz
 , PGOBLIN_JOBTABSIZ                                            // jobtabsiz
 , PGOBLIN_STYLETBSZ                                            // styletbsz
 , PGOBLIN_SYMREGS                                              // symparam
 , PGOBLIN_BINREGS                                              // binparam
 , PGOBLIN_REGSIZE                                              // regsize
+, 0
 , PGOBLIN_max                                                  // max
+, 0
+#if BLINAP > 4
+, 0
+#endif
+, outable                                                      // outable
+, iotable                                                      // iotable
+, intable                                                      // intable
 , dbasetable                                                   // dbasetable
 , jobtable                                                     // jobtable
 , styletable                                                   // styletable
 , &cmds                                                        // cmds
 , &sqlkeywords                                                 // sqlkeywords
 , {PGOBLIN_ARG1S, PGOBLIN_ARG2S, PGOBLIN_ARG3S, PGOBLIN_ARG4S} // binparams
-, { { PGOBLIN_OUTSET , PGOBLIN_ANYIVL ,  0, 0, "out"}          // Po,F  p[0].sym .val .bin .mask .name
-  , { PGOBLIN_CTL    , PGOBLIN_ANYIVL ,  1, 0, "ctl"}          // Id,F  p[1]
-  , { PGOBLIN_ISIN   , PGOBLIN_ANYIVL ,  3, 0, "in "}          // Ii,F  p[2]
-  , { PGOBLIN_CONNECT, PGOBLIN_CONNECT,  2, 0, "db "}          // Ic,Ic p[3]
-  , { PGOBLIN_JOB    , PGOBLIN_JOB    ,  2, 0, "job"}          // Ij,Ij p[4]
-  , { PGOBLIN_STYLE  , PGOBLIN_STYLE  ,~12, 3, "sty"}          // Is,Is p[5]
+#if BLINAP > 4
+, 0
+#endif
+, { { PGOBLIN_OUTSET
+    , PGOBLIN_ANYIVL
+    , 0
+    , 0
+#if BLINAP > 4
+    , 0
+#endif
+    , "out"
+    }                                                          // Po,F  p[0].sym .val .bin .mask .name
+  , { PGOBLIN_CTL
+    , PGOBLIN_ANYIVL
+    , 1
+    , 0
+#if BLINAP > 4
+    , 0
+#endif
+    , "ctl"
+    }                                                          // Id,F  p[1]
+  , { PGOBLIN_ISIN
+    , PGOBLIN_ANYIVL
+    , 3
+    , 0
+#if BLINAP > 4
+    , 0
+#endif
+    , "in "
+    }                                                          // Ii,F  p[2]
+  , { PGOBLIN_CONNECT
+    , PGOBLIN_CONNECT
+    , 2
+    , 0
+#if BLINAP > 4
+    , 0
+#endif
+    , "db "
+    }                                                          // Ic,Ic p[3]
+  , { PGOBLIN_JOB
+    , PGOBLIN_JOB
+    , 2
+    , 0
+#if BLINAP > 4
+    , 0
+#endif
+    , "job"
+    }                                                          // Ij,Ij p[4]
+  , { PGOBLIN_STYLE
+    , PGOBLIN_STYLE
+    , ~12
+    , 3
+#if BLINAP > 4
+    , 0
+#endif
+    , "sty"
+    }                                                          // Is,Is p[5]
   }                                                            // p
 /*
  *  a[].flags       I.odicjs,    .
@@ -205,14 +228,206 @@ const pgoblin_syntax pgoblin =
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_pusho  , "pusho"    } /* 008 */
+//   
+//a
+// IO
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//  ()   .
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_pushc  , "pushc"    } /* 009 */
+//   
+//a
+// CONN
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//  ()   .
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_pushj  , "pushj"    } /* 00A */
+//   
+//a
+// JOB
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//  ()   .
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_pushs  , "pushs"    } /* 00B */
+//   
+//a
+// STY
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//  ()   .
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_popo   , "popo"     } /* 00C */
+//   
+//a
+// IO
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//     (  ).
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_popc   , "popc"     } /* 00D */
+//   
+//a
+// CONN
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//     (  ).
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_popj   , "popj"     } /* 00E */
+//   
+//a
+// JOB
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//     (  ).
+  , {       Id|      Is|0x3000, {    0,    Pt,     0,   Ps}, pgoblin_pops   , "pops"     } /* 00F */
+//   
+//a
+// STY
+//a
+// ,         
+//       ,     .
+// 
+//a
+// STY
+//a
+// 0,     
+//a
+// CTL
+//a
+// ,       
+//a
+// CTL
+//a
+//   
+//a
+// STY
+//a
+// .         
+//     (  ).
 
   , {       Id|   Ic          , {    0,    Pt, Po|Pc,    0}, pgoblin_connect, "connect"  } /* 010 */
 //  ,  
@@ -232,8 +447,19 @@ const pgoblin_syntax pgoblin =
 //         ,
 //      ,    ..
   , {    Io|Id|Ii|Ic          , {Po|Pq,    Pt,    Pc,   Pt}, pgoblin_listen , "listen"   } /* 011 */
-//  notify  ,   CTL.    ,
-//     IN.  NOTIFY  
+//  notify  ,  
+//a
+// CTL
+//a
+// .    ,    
+//a
+// IN
+//a
+// . 
+//a
+// NOTIFY
+//a
+//  
 //a
 // Po
 //a
@@ -263,7 +489,10 @@ const pgoblin_syntax pgoblin =
 // JOB
 //a
 // ,   
-// CTL,      
+//a
+// CTL
+//a
+// ,      
 //a
 // Pq
 //a
@@ -310,7 +539,7 @@ const pgoblin_syntax pgoblin =
 //a
 // )  ,   .
   , { Fx                      , {    0,     0,     0,    0}, pgoblin_call   , "call"     } /* 014 */
-  , {       Id                , {    0,    Pt,     0,    0}, pgoblin_eval   , "eval"     } /* 015 */
+  , {       Id|Ii             , {    0,    Pt,     0,   Pq}, pgoblin_eval   , "eval"     } /* 015 */
 //a
 // CTL
 //a
@@ -320,8 +549,11 @@ const pgoblin_syntax pgoblin =
 //a
 // .
 //          ,
-//     #trap.
-//  
+//    
+//a
+// #trap
+//a
+// .  
 //a
 // #trap
 //a
@@ -330,14 +562,40 @@ const pgoblin_syntax pgoblin =
 //a
 // #trap
 //a
-//   .
-//  
+//   .  
 //a
 // #trap
 //a
 //  ,  
 //          .
-  , {       Id|Ii|   Is|0x2000, {    0,    Pt,    Ps,   Pt}, pgoblin_style  , "style"    } /* 016 */
+// 
+//a
+// IN
+//a
+// 
+//a
+// 0
+//a
+// , 
+//a
+// CTL
+//a
+//     
+//a
+// IN
+//a
+// ,    
+//      . 
+//  ,   
+//a
+// IN
+//a
+// , 
+//a
+// CTL
+//a
+//    .
+  , {    Io|Id|Ii|   Is|0x2000, {Po|Pm,    Pt,    Ps,   Pt}, pgoblin_style  , "style"    } /* 016 */
 //   
 //a
 // STY
@@ -352,7 +610,11 @@ const pgoblin_syntax pgoblin =
 // IN
 //a
 // .
-//  0 
+// 
+//a
+// 0
+//a
+//  
 //a
 // STY
 //a
@@ -398,9 +660,8 @@ const pgoblin_syntax pgoblin =
 // 2 -         
 //u
 //     0.
-  , {    Io|Id|   Ij|Is|0x3000, {Po|Pt,    Pt,    Pj,   Ps}, pgoblin_echo   , "echo"     } /* 018 */
-//   STY 0,   OUT    CTL,
-//  
+  , {    Io|Id|      Is|0x3000, {Po|Pt,    Pt,     0,   Ps}, pgoblin_echo   , "echo"     } /* 018 */
+//   STY 0,   OUT    CTL,  
 //a
 // OUT
 //a
@@ -416,9 +677,9 @@ const pgoblin_syntax pgoblin =
   , {    Io|Id|      Is|0x3000, {Po|Pf,    Pt,     0,   Ps}, pgoblin_file   , "file"     } /* 019 */
 //  
 //a
-// STY
+// STY 0
 //a
-// 0,  
+// ,  
 //a
 // OUT
 //a
@@ -443,9 +704,9 @@ const pgoblin_syntax pgoblin =
   , {    Io|Id|      Is|0x3000, {Po|Pf,    Pt,     0,   Ps}, pgoblin_cat    , "cat"      } /* 01A */
 //  
 //a
-// STY
+// STY 0
 //a
-// 0,  
+// ,  
 //a
 // OUT
 //a
@@ -476,8 +737,11 @@ const pgoblin_syntax pgoblin =
 //a
 // #return
 //a
-// .
-//    0 
+// .   
+//a
+// 0
+//a
+//  
 //a
 // CON
 //a
@@ -530,8 +794,7 @@ const pgoblin_syntax pgoblin =
 //a
 //  CON
 //a
-//   ,
-//    
+//   ,    
 //a
 // Pq
 //a
@@ -539,10 +802,9 @@ const pgoblin_syntax pgoblin =
 //a
 //  CTL Pq
 //a
-//  
-//  ,  0   .    
-//  ,      .
-//  
+//    ,  0   .
+//      ,
+//      .  
 //a
 // OUT
 //a
@@ -557,21 +819,34 @@ const pgoblin_syntax pgoblin =
 // OUT.
   , {    Io|Id|Ii|Ic          , {Po|Pq, Pt|Pq,    Pc,   Pq}, pgoblin_select , "select"   } /* 022 */
 //       
-// OUT.   
-// IN,    
+//a
+// OUT
+//a
+// .   
+//a
+// IN
+//a
+// ,    
 //a
 // IN Pq
 //a
 //  
 //    .   
-// CONN.
-  , {    Io|Id|Ij|Ic|Is|0x3000, {Po|Pm, Pt|Pq,    Pc,   Ps}, pgoblin_binsel , "binselect"} /* 023 */
-//   CTL     
+//a
+// CONN
+//a
+// .
+  , {    Io|Id|Ij|Ic|Is|0x3000, {Po|Pm,    Pt,    Pc,   Ps}, pgoblin_binsel , "binselect"} /* 023 */
+//  
+//a
+// CTL
+//a
+//     
 //a
 // OUT
 //a
 //   STY.
-  , {    Io|Id|Ij|Ic|Is|0x3000, {Po|Pm, Pt|Pq,    Pc,   Ps}, pgoblin_strsel , "strselect"} /* 024 */
+  , {    Io|Id|Ij|Ic|Is|0x3000, {Po|Pm,    Pt,    Pc,   Ps}, pgoblin_strsel , "strselect"} /* 024 */
 //  
 //a
 // CTL
@@ -641,12 +916,7 @@ const pgoblin_syntax pgoblin =
 // CTL
 //a
 //     ,     .
-// 
-//a
-// IN
-//a
-// 0,      1      ,
-//     IN. 
+//         IN. 
 //a
 // OUT
 //a
@@ -712,7 +982,15 @@ const pgoblin_syntax pgoblin =
 //a
 // Bpars_NONU
 //a
-// , 1 -  , 0 - .
+// ,
+//a
+// 1
+//a
+// -  ,
+//a
+// 0
+//a
+// - .
 //  ,  
 //a
 // Bpars_SYME | Bpars_ESYM | Bpars_EPRM
@@ -782,13 +1060,11 @@ const pgoblin_syntax pgoblin =
 //a
 // CON
 //a
-// ,     .
-//     
+// ,     .     
 //a
 // END
 //a
-//   ,
-//    .  
+//   ,    .  
 //a
 // #begin
 //a
@@ -796,8 +1072,7 @@ const pgoblin_syntax pgoblin =
 //a
 // #end
 //a
-//  
-// 
+//   
 //a
 // BEGIN
 //a
@@ -805,20 +1080,22 @@ const pgoblin_syntax pgoblin =
 //a
 // END
 //a
-//    .    
-//       ,      PostgreSQL.
-//  
+//    .      
+//     ,      PostgreSQL.  
 //a
 // OUT
 //a
-//  0,   ,      .
+// 
+//a
+// 0
+//a
+// ,   ,      .
   , {    Io|Id|   Ic          , {Po|Pq,    Pt,    Pc,    0}, pgoblin_end    , "end"      } /* 031 */
 //      
 //a
 // #begin
 //a
-// ,   ,
-//     .
+// ,   ,     .
 //    
 //a
 // END
@@ -827,8 +1104,7 @@ const pgoblin_syntax pgoblin =
 //a
 // CON
 //a
-// .
-//     
+// .     
 //a
 // END
 //a
@@ -836,15 +1112,100 @@ const pgoblin_syntax pgoblin =
 //a
 // OUT
 //a
-//  0,   ,      .
+// 
+//a
+// 0
+//a
+// ,   ,      .
   , {    Io|Id|   Ic          , {Po|Pq,    Pt,    Pc,    0}, pgoblin_rollbk , "rollback" } /* 032 */
 //  ROLLBACK.  
 //a
 // OUT
 //a
-//  0,   ,      .
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
-  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
+// 
+//a
+// 0
+//a
+// ,   ,      .
+  , { Ff|Io|Id|Ii|   Ij|    Fc, {Po|Pm,    Pq,    Pj,   Pm}, pgoblin_fork   , "fork"     } /* 033 */
+//a
+// pgoblin
+//a
+// ,     
+//a
+// #fork
+//a
+// 
+//a
+// #exit
+//a
+// 
+//a
+// #return
+//a
+// ,
+//   ,     .
+//         ,
+//     .
+//        ,
+//           .
+//         
+//    ,     
+//  . 
+//a
+// OUT
+//a
+//          ,  
+//a
+// IN
+//a
+//          ,
+//   . 
+//a
+// CTL
+//a
+// 
+//a
+// 0
+//a
+// ,    ,   
+//a
+// CTL
+//a
+// ,      
+//a
+// CTL
+//a
+//       
+//a
+// CTL
+//a
+// .
+  , {    Io|Id|Ii|   Ij|    Fc, {Po   ,    Pt,    Pj,   Pq}, pgoblin_out    , "out"      } /* 034 */
+//  
+//a
+// CTL
+//a
+//    
+//a
+// OUT
+//a
+// . 
+//a
+// IN
+//a
+// 
+//a
+// 0
+//a
+// ,         .      
+//a
+// CTL
+//a
+// .    .     
+//a
+// JOB
+//a
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
@@ -1061,44 +1422,9 @@ const pgoblin_syntax pgoblin =
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
 
-  , {    Io|Id|   Ic          , {Po|Pq,    Pt,    Pc,    0}, pgoblin_begin  , "begin"    } /* 100 */
-  , {    Io|Id|   Ic          , {Po|Pq,    Pt,    Pc,    0}, pgoblin_end    , "end"      } /* 101 */
-  , { Fl|Io|   Ii|   Ij    |Ff, {Po|Pm,     0,    Pj,   Pm}, pgoblin_fork   , "fork"     } /* 102 */
-//a
-// pgoblin
-//a
-// ,     
-//a
-// #fork
-//a
-// 
-//a
-// #exit
-//a
-// 
-//a
-// #return
-//a
-// ,
-//   ,     .
-//         ,
-//     .
-//        ,
-//           .
-//         
-//    ,     
-//  .
-// 
-//a
-// OUT
-//a
-//          ,
-//  
-//a
-// IN
-//a
-//          ,
-//   .
+  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL} /* 100 */
+  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL} /* 101 */
+  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL} /* 102 */
   , { Fl|            Ij       , {    0,     0,    Pj,    0}, pgoblin_wait   , "wait"     } /* 103 */
 //    ,  
 //a
@@ -1119,13 +1445,11 @@ const pgoblin_syntax pgoblin =
 //a
 // CTL
 //a
-// .
-//    
+// .    
 //a
 // #trap
 //a
-// ,       ,
-//  
+// ,       ,  
 //a
 // CON
 //a
@@ -1142,7 +1466,11 @@ const pgoblin_syntax pgoblin =
 // IN
 //a
 // .      
-//     . 0  ,   .
+//     .
+//a
+// 0
+//a
+//   ,   .
   , { Fl|Io|   Ii|Ic|Is|0x1000, {   Pm, Po|Ps, Po|Pc,Po|Pm}, pgoblin_close  , "close"    } /* 108 */
 //   
 //a
@@ -1169,13 +1497,15 @@ const pgoblin_syntax pgoblin =
 //a
 // OUT
 //a
-// .
-//      0   .
-//  
+// .      0   .  
 //a
 // CON
 //a
-// ,   0,  ,    
+// ,  
+//a
+// 0
+//a
+// ,  ,    
 //      .
   , { Fl|Io|      Ij|Is|0x3000, {Po|Pt,     0, Po|Pj,Po|Ps}, pgoblin_clear  , "clear"    } /* 109 */
 // 
@@ -1186,7 +1516,15 @@ const pgoblin_syntax pgoblin =
 //a
 // STY
 //a
-// ,   0. 0
+// ,  
+//a
+// 0
+//a
+// .
+//a
+// 0
+//a
+// 
 //a
 // JOB
 //a
@@ -1194,8 +1532,11 @@ const pgoblin_syntax pgoblin =
 //a
 // STY
 //a
-//  .
-//     0
+//  .    
+//a
+// 0
+//a
+// 
 //a
 // OUT
 //a
@@ -1212,7 +1553,7 @@ const pgoblin_syntax pgoblin =
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
   , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL}
 
-  , { Fl|Io|         Ij       , {Po|Pm,     0,    Pj,    0}, pgoblin_getarg , "getarg"   } /* 110 */
+  , { Fx                      , {    0,     0,     0,    0}, NULL           ,        NULL} /* 110 */
   , { Fl|Io|         Ij       , {Po|Pm,     0,    Pj,    0}, pgoblin_getenv , "getenv"   } /* 111 */
 //  OUT     COPY  3 :
 //n
@@ -1242,9 +1583,7 @@ const pgoblin_syntax pgoblin =
 //a
 // GET
 //a
-// ,   
-//a
-// getCGIparm.
+// .
   , { Fl|Io|   Ii|   Ij       , {Po|Pm,     0,    Pj,   Pm}, pgoblin_getpost, "getpost"  } /* 113 */
 // 
 //a
@@ -1264,8 +1603,7 @@ const pgoblin_syntax pgoblin =
 //a
 // IN
 //a
-// .
-//     
+// .     
 //a
 // CGI
 //a
@@ -1273,9 +1611,7 @@ const pgoblin_syntax pgoblin =
 //a
 // POST
 //a
-// ,   
-//a
-// getCGIparm.
+// .
   , { Fl|Io|         Ij       , {Po|Pm,     0,    Pj,    0}, pgoblin_getproc, "getproc"  } /* 114 */
 // 
 //a
@@ -1291,9 +1627,8 @@ const pgoblin_syntax pgoblin =
 // -  ,   0
 // -  
 //n
-//     .  
-//a
-// /proc/.
+//     .
+//  0   exec()     -1.
   , { Fl|Io|         Ij       , {Po|Pm,     0,     0,    0}, pgoblin_getjob , "getjob"   } /*!ch[2] */
 // 
 //a
@@ -1320,9 +1655,9 @@ const pgoblin_syntax pgoblin =
 //a
 //  
 //a
-// JOB
+// JOB 0
 //a
-// 0.
+// .
   , { Fx                      , {    0,     0,     0,    0}, pgoblin_getspq , "getspq"   } /* 116 */
   , { Fl|Io|   Ii             , {Po|Pm,     0,     0,   Pq}, pgoblin_getmeta, "getmeta"  } /* 117 */
 // 
@@ -1352,6 +1687,8 @@ const pgoblin_syntax pgoblin =
 // IN
 //a
 //      NULL.
-// 2    NULL    .
+//     NULL    .
   , { Fl|Io|   Ii             , {Po|Pq,     0,     0,   Pq}, pgoblin_info   , "info"     } /* 118 */
+  , { Fl|Io                   , {Po|Pm,     0,     0,    0}, pgoblin_dump   , "dump"     } /* 119 */
+//   .
 } };
