@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2017..2025 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,15 +24,15 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2017..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: func.c,v 1.68 2022/03/26 23:07:14 babolo Exp $"
+#ident "@(#) Copyright (C)2017..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: func.c,v 1.93 2025/01/25 00:12:13 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define RECOBE_COMPAT    4
-#define PGOBLIN_COMPAT   4
+#define RECOBE_COMPAT    6
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -54,7 +54,7 @@
 #include <mife.h>
 #include "pgoblin.h"
 
-static int16_t          onecopy[16] = {-1, 0, 0, 0,  0, 0, 0, 0,  'b', 't', 'n', 'v',  'f', 'r', 0, 0};
+static int16_t onecopy[16] = {-1, 0, 0, 0,  0, 0, 0, 0,  'b', 't', 'n', 'v',  'f', 'r', 0, 0};
 
 int
 pgoblin_onecsym(char in) {
@@ -70,7 +70,7 @@ ssize_t
  **  end < 0, ,  end < 0,  \t,  end >= 0,           **
  **    .                                            **
  **  len < 0,  in   \0,                                **
- **   len   in.                                    **
+ **   len   in.                                   **
  *************************************************************************
  **                                                                     **/
 pgoblin_onecopy(char *escaped, const char *in, ssize_t len, int end) { /**
@@ -94,9 +94,9 @@ pgoblin_onecopy(char *escaped, const char *in, ssize_t len, int end) { /**
                 break;
             default:
                 escaped[ex++] = '\\';
-                escaped[ex++] = c;
+                escaped[ex++] = (char)c;
     }   }   }
-    for (int c = (0 > end) ? ~end : end; c > 0; --c) {
+    for (c = (0 > end) ? ~end : end; c > 0; --c) {
         escaped[ex++] = '\t';
         escaped[ex++] = '\\';
         escaped[ex++] = 'N';
@@ -129,7 +129,7 @@ pgoblin_breg(pgoblin_command cmd, pgoblin_nr r) {                     /**
             /*      */
           : pgoblin.p[r].bin
         ;
-        if  (!(pgoblin.a[cmd].ch[j] & pgoblin.p[r].val) && !!pgoblin.a[cmd].ch[j]) {
+        if  ((PGO_CJOB == r) && !(pgoblin.a[cmd].ch[j] & pgoblin.p[r].val) && !!pgoblin.a[cmd].ch[j]) {
             /*                            *
              *     ,  -     . *
              *    0 j ,   j  ,      *
@@ -144,11 +144,12 @@ pgoblin_rio *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_upio(pgoblin_main *options, pgoblin_nr nio) {               /**
+pgoblin_upio(pgoblin_exenv *exenv, pgoblin_nr nio) {                /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = (u_int16_t)-1;
     pgoblin_rio *prev = NULL;
     pgoblin_rio *rio  = NULL;
 
@@ -158,19 +159,24 @@ pgoblin_upio(pgoblin_main *options, pgoblin_nr nio) {               /**
         errno = EDOM;
         goto out;
     }
-    if  (!((1ULL << nio) & options->useio)) {
-        ifBLIN_QX4("IO reg %u masked+", pgoblin_regn[nio]);
-        prev = options->io[nio];
-        options->io[nio] = NULL;
+    if  (!((1ULL << nio) & exenv->options->useio)) {
+        ifBLIN_QX6("IO reg %u masked+", pgoblin_regn[nio]);
+        prev = exenv->options->io[nio];
+        exenv->options->io[nio] = NULL;
+        if  (!!prev) deep = prev->deep;
     }
-    if  (!(rio = options->io[nio])) {
-        if  (!(options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
-            ifBLIN_QW0("No mem IO %c", pgoblin_regn[nio]);
-            goto out;
-        }
-        rio->prev = prev;
-        rio->flags = options->flags & BLIN_MASK;
-        options->useio |= (1ULL << nio);
+    if  (!(rio = exenv->options->io[nio])) {
+        MARK_IO_GO(nio);;
+        ;;  if  (!(exenv->options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
+        ;;      ifBLIN_QW0("No mem IO %c", pgoblin_regn[nio]);
+        ;;      goto out;
+        ;;  }
+        ;;  rio->prev = prev;
+        ;;  rio->deep = ++deep;
+        ;;  rio->nio = nio;
+        ;;  rio->flags = exenv->options->flags & BLIN_MASK;
+        MARK_IO_WENT(nio);;
+        exenv->options->useio |= (1ULL << nio);
     }
 out:
     return(rio);
@@ -181,11 +187,12 @@ pgoblin_rdb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_upcon(pgoblin_main *options, pgoblin_nr ndb)          {     /**
+pgoblin_upcon(pgoblin_exenv *exenv, pgoblin_nr ndb) {               /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = (u_int16_t)-1;
     pgoblin_rdb *prev = NULL;
     pgoblin_rdb *rdb  = NULL;
 
@@ -194,19 +201,24 @@ pgoblin_upcon(pgoblin_main *options, pgoblin_nr ndb)          {     /**
         errno = EDOM;
         goto out;
     }
-    if  (!((1ULL << ndb) & options->usecon)) {
-        ifBLIN_QX4("CONN %c masked+", pgoblin_regn[ndb]);
-        prev = options->conn[ndb];
-        options->conn[ndb] = NULL;
+    if  (!((1ULL << ndb) & exenv->options->usecon)) {
+        ifBLIN_QX6("CONN %c masked+", pgoblin_regn[ndb]);
+        prev = exenv->options->conn[ndb];
+        exenv->options->conn[ndb] = NULL;
+        if  (!!prev) deep = prev->deep;
     }
-    if  (!(rdb = options->conn[ndb])) {
-        if  (!(options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
-            ifBLIN_QW0("No mem CONN %c", pgoblin_regn[ndb]);
-            goto out;
-        }
-        rdb->prev = prev;
-        rdb->flags = options->flags & BLIN_MASK;
-        options->usecon |= (1ULL << ndb);
+    if  (!(rdb = exenv->options->conn[ndb])) {
+        MARK_CONN_GO(ndb);;
+        ;;  if  (!(exenv->options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
+        ;;      ifBLIN_QW0("No mem CONN %c", pgoblin_regn[ndb]);
+        ;;      goto out;
+        ;;  }
+        ;;  rdb->prev = prev;
+        ;;  rdb->deep = ++deep;
+        ;;  rdb->ndb = ndb;
+        ;;  rdb->flags = exenv->options->flags & BLIN_MASK;
+        MARK_CONN_WENT(ndb);;
+        exenv->options->usecon |= (1ULL << ndb);
     }
 out:
     return(rdb);
@@ -217,31 +229,38 @@ pgoblin_rjb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_upjob(pgoblin_main *options, pgoblin_nr njb) {              /**
+pgoblin_upjob(pgoblin_exenv *exenv, pgoblin_nr njb) {               /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = (u_int16_t)-1;
     pgoblin_rjb *prev = NULL;
-    pgoblin_rjb *rjb = NULL;
+    pgoblin_rjb *rjb  = NULL;
 
     if  (pgoblin.regsize <= njb) {
         ifBLIN_QX0("Ill JOB reg num %u", njb);
+        errno = EDOM;
         goto out;
     }
-    if  (!((1ULL << njb) & options->usejob)) {
-        ifBLIN_QX4("JOB %c masked+", pgoblin_regn[njb]);
-        prev = options->job[njb];
-        options->job[njb] = NULL;
+    if  (!((1ULL << njb) & exenv->options->usejob)) {
+        ifBLIN_QX6("JOB %c masked+", pgoblin_regn[njb]);
+        prev = exenv->options->job[njb];
+        exenv->options->job[njb] = NULL;
+        if  (!!prev) deep = prev->deep;
     }
-    if  (!(rjb = options->job[njb])) {
-        if  (!(options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
-            ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
-            goto out;
-        }
-        rjb->prev = prev;
-        rjb->flags = options->flags & BLIN_MASK;
-        options->usejob |= (1ULL << njb);
+    if  (!(rjb = exenv->options->job[njb])) {
+        MARK_JOB_GO(njb);;
+        ;;  if  (!(exenv->options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
+        ;;      ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
+        ;;      goto out;
+        ;;  }
+        ;;  rjb->prev = prev;
+        ;;  rjb->deep = ++deep;
+        ;;  rjb->njb = njb;
+        ;;  rjb->flags = exenv->options->flags & BLIN_MASK;
+        MARK_JOB_WENT(njb);;
+        exenv->options->usejob |= (1ULL << njb);
     }
 out:
     return(rjb);
@@ -252,31 +271,38 @@ pgoblin_rst *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_upsty(pgoblin_main *options, pgoblin_nr nst) {              /**
+pgoblin_upsty(pgoblin_exenv *exenv, pgoblin_nr nst) {               /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = (u_int16_t)-1;
     pgoblin_rst *prev = NULL;
     pgoblin_rst *rst  = NULL;
 
     if  (pgoblin.regsize <= nst) {
         ifBLIN_QX0("Ill STY reg num %u", nst);
+        errno = EDOM;
         goto out;
     }
-    if  (!((1ULL << nst) & options->usesty)) {
-        ifBLIN_QX4("STY %c masked+", pgoblin_regn[nst]);
-        prev = options->style[nst];
-        options->style[nst] = NULL;
+    if  (!((1ULL << nst) & exenv->options->usesty)) {
+        ifBLIN_QX6("STY %c masked+", pgoblin_regn[nst]);
+        prev = exenv->options->style[nst];
+        exenv->options->style[nst] = NULL;
+        if  (!!prev) deep = prev->deep;
     }
-    if  (!(rst = options->style[nst])) {
-        if  (!(options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
-            ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
-            goto out;
-        }
-        rst->prev = prev;
-        rst->flags = options->flags & BLIN_MASK;
-        options->usesty |= (1ULL << nst);
+    if  (!(rst = exenv->options->style[nst])) {
+        MARK_STYLE_GO(nst);;
+        ;;  if  (!(exenv->options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
+        ;;      ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
+        ;;      goto out;
+        ;;  }
+        ;;  rst->prev = prev;
+        ;;  rst->deep = ++deep;
+        ;;  rst->nst = nst;
+        ;;  rst->flags = exenv->options->flags & BLIN_MASK;
+        MARK_STYLE_WENT(nst);;
+        exenv->options->usesty |= (1ULL << nst);
     }
 out:
     return(rst);
@@ -292,7 +318,6 @@ pgoblin_meio(pgoblin_main *options, pgoblin_nr nio) {               /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    pgoblin_rio *prev = NULL;
     pgoblin_rio *rio  = NULL;
 
 
@@ -302,8 +327,7 @@ pgoblin_meio(pgoblin_main *options, pgoblin_nr nio) {               /**
         goto out;
     }
     if  (!((1ULL << nio) & options->useio)) {
-        ifBLIN_QX4("IO %c masked", pgoblin_regn[nio]);
-        prev = options->io[nio];
+        ifBLIN_QX6("IO %c masked", pgoblin_regn[nio]);
         errno = ENOENT;
         goto out;
     }
@@ -325,7 +349,6 @@ pgoblin_mecon(pgoblin_main *options, pgoblin_nr ndb) {              /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    pgoblin_rdb *prev = NULL;
     pgoblin_rdb *rdb  = NULL;
 
     if  (pgoblin.regsize <= ndb) {
@@ -334,8 +357,7 @@ pgoblin_mecon(pgoblin_main *options, pgoblin_nr ndb) {              /**
         goto out;
     }
     if  (!((1ULL << ndb) & options->usecon)) {
-        ifBLIN_QX4("CONN %c masked", pgoblin_regn[ndb]);
-        prev = options->conn[ndb];
+        ifBLIN_QX6("CONN %c masked", pgoblin_regn[ndb]);
         errno = ENOENT;
         goto out;
     }
@@ -357,16 +379,15 @@ pgoblin_mejob(pgoblin_main *options, pgoblin_nr njb) {              /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    pgoblin_rjb *prev = NULL;
     pgoblin_rjb *rjb = NULL;
 
     if  (pgoblin.regsize <= njb) {
         ifBLIN_QX0("Ill JOB reg num %u", njb);
+        errno = EDOM;
         goto out;
     }
     if  (!((1ULL << njb) & options->usejob)) {
-        ifBLIN_QX4("JOB %c masked", pgoblin_regn[njb]);
-        prev = options->job[njb];
+        ifBLIN_QX6("JOB %c masked", pgoblin_regn[njb]);
         errno = ENOENT;
         goto out;
     }
@@ -388,16 +409,15 @@ pgoblin_mesty(pgoblin_main *options, pgoblin_nr nst) {              /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    pgoblin_rst *prev = NULL;
     pgoblin_rst *rst  = NULL;
 
     if  (pgoblin.regsize <= nst) {
         ifBLIN_QX0("Ill STY reg num %u", nst);
+        errno = EDOM;
         goto out;
     }
     if  (!((1ULL << nst) & options->usesty)) {
-        ifBLIN_QX4("STY %c masked", pgoblin_regn[nst]);
-        prev = options->style[nst];
+        ifBLIN_QX6("STY %c masked", pgoblin_regn[nst]);
         errno = ENOENT;
         goto out;
     }
@@ -414,33 +434,39 @@ pgoblin_rio *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushio(pgoblin_main *options, pgoblin_nr nio) {             /**
+pgoblin_pushio(pgoblin_exenv *exenv, pgoblin_nr nio) {              /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = 0;
     pgoblin_rio *prev = NULL;
     pgoblin_rio *rio  = NULL;
 
-
     if  (pgoblin.regsize <= nio) {
         ifBLIN_QX0("Ill IO reg num %u", nio);
         errno = EDOM;
         goto out;
     }
-    prev = options->io[nio];
-    if  (!(options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
-        ifBLIN_QW0("No mem for IO %c", pgoblin_regn[nio]);
-        goto out;
-    }
-    rio->prev = prev;
-    rio->flags = options->flags & BLIN_MASK;
-    options->useio |= (1ULL << nio);
+    prev = exenv->options->io[nio];
+    ifBLIN_QX6("IO   %u(%c) ->%"BLIN_X, nio, pgoblin_regn[nio], BLIN_I(prev));
+    if  (!!prev) deep = prev->deep;
+    MARK_IO_GO(nio);;
+    ;;  if  (!(exenv->options->io[nio] = rio = calloc(1, sizeof(pgoblin_rio)))) {
+    ;;      ifBLIN_QW0("No mem for IO %c", pgoblin_regn[nio]);
+    ;;      goto out;
+    ;;  }
+    ;;  rio->prev = prev;
+    ;;  rio->deep = ++deep;
+    ;;  rio->nio = nio;
+    ;;  rio->flags = exenv->options->flags & BLIN_MASK;
+    MARK_IO_WENT(nio);;
+    exenv->options->useio |= (1ULL << nio);
+    ifBLIN_QX6("IO   %08X N%u (%u)", rio->flags, rio->nio, rio->deep);
 out:
-    ifBLIN_QX6( "IO   %"BLIN_X"->%"BLIN_X"=%s~"
+    ifBLIN_QX6( "IO   %"BLIN_X"->%"BLIN_X
               , BLIN_I(rio)
               , BLIN_I(prev)
-              , (rio && rio->text) ? rio->text : ""
               );
     return(rio);
 #   undef blin_internal_flags
@@ -450,11 +476,12 @@ pgoblin_rdb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushcon(pgoblin_main *options, pgoblin_nr ndb) {            /**
+pgoblin_pushcon(pgoblin_exenv *exenv, pgoblin_nr ndb) {             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = 0;
     pgoblin_rdb *prev = NULL;
     pgoblin_rdb *rdb  = NULL;
 
@@ -463,14 +490,19 @@ pgoblin_pushcon(pgoblin_main *options, pgoblin_nr ndb) {            /**
         errno = EDOM;
         goto out;
     }
-    prev = options->conn[ndb];
-    if  (!(options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
-        ifBLIN_QW0("No mem for CONN %c", pgoblin_regn[ndb]);
-        goto out;
-    }
-    rdb->prev = prev;
-    rdb->flags = options->flags & BLIN_MASK;
-    options->usecon |= (1ULL << ndb);
+    prev = exenv->options->conn[ndb];
+    if  (!!prev) deep = prev->deep;
+    MARK_CONN_GO(ndb);;
+    ;;  if  (!(exenv->options->conn[ndb] = rdb = calloc(1, sizeof(pgoblin_rdb)))) {
+    ;;      ifBLIN_QW0("No mem for CONN %c", pgoblin_regn[ndb]);
+    ;;      goto out;
+    ;;  }
+    ;;  rdb->prev = prev;
+    ;;  rdb->deep = ++deep;
+    ;;  rdb->ndb = ndb;
+    ;;  rdb->flags = exenv->options->flags & BLIN_MASK;
+    MARK_CONN_WENT(ndb);;
+    exenv->options->usecon |= (1ULL << ndb);
 out:
     ifBLIN_QX6("CONN %"BLIN_X"->%"BLIN_X, BLIN_I(rdb), BLIN_I(prev));
     return(rdb);
@@ -481,26 +513,33 @@ pgoblin_rjb *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushjob(pgoblin_main *options, pgoblin_nr njb) {            /**
+pgoblin_pushjob(pgoblin_exenv *exenv, pgoblin_nr njb) {             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = 0;
     pgoblin_rjb *prev = NULL;
     pgoblin_rjb *rjb = NULL;
 
     if  (pgoblin.regsize <= njb) {
         ifBLIN_QX0("Ill JOB reg num %u", njb);
+        errno = EDOM;
         goto out;
     }
-    prev = options->job[njb];
-    if  (!(options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
-        ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
-        goto out;
-    }
-    rjb->prev = prev;
-    rjb->flags = options->flags & BLIN_MASK;
-    options->usejob |= (1ULL << njb);
+    prev = exenv->options->job[njb];
+    if  (!!prev) deep = prev->deep;
+    MARK_JOB_GO(njb);;
+    ;;  if  (!(exenv->options->job[njb] = rjb = calloc(1, sizeof(pgoblin_rjb)))) {
+    ;;      ifBLIN_QW0("No mem for JOB %c", pgoblin_regn[njb]);
+    ;;      goto out;
+    ;;  }
+    ;;  rjb->prev = prev;
+    ;;  rjb->deep = ++deep;
+    ;;  rjb->njb = njb;
+    ;;  rjb->flags = exenv->options->flags & BLIN_MASK;
+    MARK_JOB_WENT(njb);;
+    exenv->options->usejob |= (1ULL << njb);
 out:
     ifBLIN_QX6("JOB  %"BLIN_X"->%"BLIN_X, BLIN_I(rjb), BLIN_I(prev));
     return(rjb);
@@ -511,26 +550,33 @@ pgoblin_rst *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_pushsty(pgoblin_main *options, pgoblin_nr nst) {            /**
+pgoblin_pushsty(pgoblin_exenv *exenv, pgoblin_nr nst) {             /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int16_t    deep = 0;
     pgoblin_rst *prev = NULL;
     pgoblin_rst *rst  = NULL;
 
     if  (pgoblin.regsize <= nst) {
         ifBLIN_QX0("Ill STY reg num %u", nst);
+        errno = EDOM;
         goto out;
     }
-    prev = options->style[nst];
-    if  (!(options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
-        ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
-        goto out;
-    }
-    rst->prev = prev;
-    rst->flags = options->flags & BLIN_MASK;
-    options->usesty |= (1ULL << nst);
+    prev = exenv->options->style[nst];
+    if  (!!prev) deep = prev->deep;
+    MARK_STYLE_GO(nst);;
+    ;;  if  (!(exenv->options->style[nst] = rst = calloc(1, sizeof(pgoblin_rst)))) {
+    ;;      ifBLIN_QW0("No mem for STY %c", pgoblin_regn[nst]);
+    ;;      goto out;
+    ;;  }
+    ;;  rst->prev = prev;
+    ;;  rst->deep = ++deep;
+    ;;  rst->nst = nst;
+    ;;  rst->flags = exenv->options->flags & BLIN_MASK;
+    MARK_STYLE_WENT(nst);;
+    exenv->options->usesty |= (1ULL << nst);
 out:
     ifBLIN_QX6("STY  %"BLIN_X"->%"BLIN_X, BLIN_I(rst), BLIN_I(prev));
     return(rst);
@@ -546,16 +592,22 @@ pgoblin_popio(pgoblin_main *options, pgoblin_nr nio) {              /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    int ex = EX_OK;
+    int          ex   = EX_OK;
     pgoblin_rio *prev = NULL;
 
     if  (pgoblin.regsize <= nio) {
         ifBLIN_QX0("Ill IO reg num %u", nio);
+        ex = EX_CANTCREAT;
+        errno = EDOM;
         goto out;
     }
-    prev = options->io[nio]->prev;
-    free(options->io[nio]); /* XXXX   XXXX */
-    options->io[nio] = prev;
+    if  (!!options->io[nio]) {
+        prev = options->io[nio]->prev;
+        MARK_IO_ALL_GO(options->io[nio]);;
+        ;;  free(options->io[nio]); /* XXXX   XXXX */
+        ;;  options->io[nio] = prev;
+        MARK_IO_ALL_WENT(nio);;
+    }
     if  (!prev) options->useio &= ~(1ULL << nio);
 out:
     ifBLIN_QX6("IO   %"BLIN_X" %u", BLIN_I(prev), ex);
@@ -572,16 +624,22 @@ pgoblin_popcon(pgoblin_main *options, pgoblin_nr ndb) {             /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    int ex = EX_OK;
+    int          ex   = EX_OK;
     pgoblin_rdb *prev = NULL;
 
     if  (pgoblin.regsize <= ndb) {
         ifBLIN_QX0("Ill CONN reg num %u", ndb);
+        ex = EX_CANTCREAT;
+        errno = EDOM;
         goto out;
     }
-    prev = options->conn[ndb]->prev;
-    free(options->conn[ndb]); /* XXXX   XXXX */
-    options->conn[ndb] = prev;
+    if  (!!options->conn[ndb]) {
+        prev = options->conn[ndb]->prev;
+        MARK_R_CONN_GO(options->conn[ndb]);;
+        ;;  free(options->conn[ndb]); /* XXXX   XXXX */
+        ;;  options->conn[ndb] = prev;
+        MARK_CONN_WENT(ndb);;
+    }
     if  (!prev) options->usecon &= ~(1ULL << ndb);
 out:
     ifBLIN_QX6("CONN %"BLIN_X" %u", BLIN_I(prev), ex);
@@ -598,16 +656,22 @@ pgoblin_popjob(pgoblin_main *options, pgoblin_nr njb) {             /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    int ex = EX_OK;
+    int          ex   = EX_OK;
     pgoblin_rjb *prev = NULL;
 
     if  (pgoblin.regsize <= njb) {
         ifBLIN_QX0("Ill JOB reg num %u", njb);
+        ex = EX_CANTCREAT;
+        errno = EDOM;
         goto out;
     }
-    prev = options->job[njb]->prev;
-    free(options->job[njb]); /* XXXX   XXXX */
-    options->job[njb] = prev;
+    if  (!!options->job[njb]) {
+        prev = options->job[njb]->prev;
+        MARK_R_JOB_GO(options->job[njb]);;
+        ;;  free(options->job[njb]); /* XXXX      XXXX */
+        ;;  options->job[njb] = prev;
+        MARK_JOB_WENT(njb);;
+    }
     if  (!prev) options->usejob &= ~(1ULL << njb);
 out:
     ifBLIN_QX6("JOB  %"BLIN_X" %u", BLIN_I(prev), ex);
@@ -624,55 +688,25 @@ pgoblin_popsty(pgoblin_main *options, pgoblin_nr nst) {             /**
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    int ex = EX_OK;
+    int          ex   = EX_OK;
     pgoblin_rst *prev = NULL;
 
     if  (pgoblin.regsize <= nst) {
         ifBLIN_QX0("Ill STY reg num %u", nst);
+        ex = EX_CANTCREAT;
+        errno = EDOM;
         goto out;
     }
-    prev = options->style[nst]->prev;
-    free(options->style[nst]); /* XXXX   XXXX */
-    options->style[nst] = prev;
+    if  (!!options->style[nst]) {
+        prev = options->style[nst]->prev;
+        MARK_R_STYLE_GO(options->style[nst]);;
+        ;;  free(options->style[nst]); /* XXXX   XXXX */
+        ;;  options->style[nst] = prev;
+        MARK_STYLE_WENT(nst);;
+    }
     if  (!prev) options->usesty &= ~(1ULL << nst);
 out:
     ifBLIN_QX6("STY  %"BLIN_X" %u", BLIN_I(prev), ex);
     return(ex);
 #   undef blin_internal_flags
 }
-
-ssize_t
-/**********************************************************************
- **********************************************************************
- **                                                                  **/
-pgoblin_wr(pgoblin_rio *rio, const void *buf, size_t size) {        /**
- **                                                                  **
- **********************************************************************
- **********************************************************************/
-#   define blin_internal_flags (rio->flags & BLIN_MASK)
-    ssize_t e;
-
-    if  (!(PGOBLIN_END & rio->flags)) {
-        if  (0 > (e = rio->wri(rio->onu, buf, size))) {
-            if  (!(PGOBLIN_TRAPDEF & rio->flags)) {
-                ifBLIN_QW0("IO out");
-            } else {
-                ifBLIN_QW0("IO out %s", (PGOBLIN_ERROR & rio->flags) ? "ignore" : "error");
-                e = 0;
-            }
-            if  (!(PGOBLIN_ERROR & rio->flags)) {
-                rio->flags |= PGOBLIN_ERROR;
-            } else {
-                rio->flags |= PGOBLIN_END;
-            }
-        } else {
-            rio->flags &= ~PGOBLIN_END;
-        }
-    } else if (!(PGOBLIN_TRAPDEF & rio->flags)) {
-        e = -1;
-    } else {
-        e = size;
-    }
-    return(e);
-#   undef blin_internal_flags
-}
