@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2024 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,15 +24,15 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: ini.c,v 1.14 2022/01/09 15:27:41 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2024 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: ini.c,v 1.41 2024/03/17 18:47:42 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define RECOBE_COMPAT    4
-#define PGOBLIN_COMPAT   4
+#define RECOBE_COMPAT    6
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -56,18 +56,20 @@
 #include <mife.h>
 #include "pgoblin.h"
 
+#define PREPLUGIN 3
+
 BLIN_flag pgoblin_default;
 static size_t u12[] = {341, 512, 512};
 
 static struct {
-    char *nm;
-    void *mod;
-} mods[PGOBLIN_SYMREGS][3] =
+    char const *nm ;
+    void       *mod;
+} mods[PGOBLIN_SYMREGS][PREPLUGIN] =
 { { {     NULL, NULL}
   , {     NULL, NULL}
   , {     NULL, NULL}
   }
- ,{ {     NULL, NULL}
+ ,{ {"0"      , &pgoblin_io_0}
   , {     NULL, NULL}
   , {     NULL, NULL}
   }
@@ -89,25 +91,25 @@ static struct {
 } };
 
 static void *
-/**************************************************************************
- **************************************************************************
- **                                                                      **/
-dload(pgoblin_main *options, int domain, int rs, const char *nm)      { /**
- **                                                                      **
- **************************************************************************
- **************************************************************************/
+/*****************************************************************************
+ *****************************************************************************
+ **                                                                         **/
+dload(pgoblin_main *options, int domain, size_t rs, const char *nm){       /**
+ **                                                                         **
+ *****************************************************************************
+ *****************************************************************************/
 #   define blin_internal_flags (options->flags & BLIN_MASK)
     char *libname = NULL;
     void *module  = NULL;
     void *handle  = NULL;
 
     for (int i = 0; !!options->libexec[i] && (PGOBLIN_SOSIZE > i); ++i) {
-        asprintf(&libname, "%s/libpgoblin%d%.*s.so", options->libexec[i], domain, rs, nm);
+        asprintf(&libname, "%s/libpgoblin%d%.*s.so", options->libexec[i], domain, (int)rs, nm);
         if  (!libname) {
             ifBLIN_QW0("No mem");
             goto out;
         }
-        if  (!(handle = dlopen(libname, /* RTLD_NOW | */ RTLD_LAZY | RTLD_LOCAL))) {
+        if  (!(handle = dlopen(libname, RTLD_LAZY | RTLD_LOCAL))) {
             ifBLIN_QX1("[%d]dlopen %s: %s", i, libname, dlerror());
         }
         free(libname);
@@ -115,12 +117,16 @@ dload(pgoblin_main *options, int domain, int rs, const char *nm)      { /**
         if  (!!handle) break;
     }
     if  (!handle) {
-        asprintf(&libname, PGOBLIN_LIBDIR"/pgoblin%u/libpgoblin%d%.*s.so", VMAJOR, domain, rs, nm);
+        asprintf( &libname, PGOBLIN_LIBDIR"/pgoblin%u/libpgoblin%d%.*s.so"
+                , VMAJOR
+                , domain
+                , (int)rs, nm
+                );
         if  (!libname) {
             ifBLIN_QW0("No mem");
             goto out;
         }
-        if  (!(handle = dlopen(libname, /* RTLD_NOW | */ RTLD_LAZY | RTLD_LOCAL))) {
+        if  (!(handle = dlopen(libname, RTLD_LAZY | RTLD_LOCAL))) {
             ifBLIN_QX0("dlopen %s: %s", libname, dlerror());
             goto out;
     }   }
@@ -138,23 +144,167 @@ int
 /**************************************************************************
  **************************************************************************
  **                                                                      **/
-pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
+pgoblin_load(pgoblin_exenv *exenv, int mode, const char *nm, int ns) {  /**
  **                                                                      **
  **************************************************************************
  **************************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
     int           loaded = -1;
-    int           slot;
-    int           wrt;
-    int           ex;
+    u_int         slot   ;
+    int           wrt    ;
+    size_t        rs     ;
+    int           ex     ;
 
-    if  (0 > rs) rs = strlen(nm);
-    ifBLIN_QX4("Load %d [%d]=%.*s", mode, rs, rs, nm);
+    if  (0 > ns) rs = strlen(nm); else rs = (size_t)ns;
+    ifBLIN_QX4("Load %d [%zu]=%.*s", mode, rs, (int)rs, nm);
+    /* PGOBLIN_MODSHIFT    , default 0 */
     switch(mode) {
+    case 0:
+        /*    ,      */
+        for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.outablesz) && !!SLOUT(slot); ++slot) {
+            ifBLIN_QX7("[%d]=%.*s", slot, (int)rs, SLOUT(slot)->name);
+            if  (!strncmp(SLOUT(slot)->name, nm, rs) && !SLOUT(slot)->name[rs]) {
+                loaded = (int)slot;
+                goto out;
+        }   }
+        if  (slot >= pgoblin.outablesz) {
+            ifBLIN_QX0("No free slots for OUT %s", nm);
+            goto out;
+        }
+        if  (PGOBLIN_MODSHIFT == slot) { /*    ,    */
+            pgoblin_rio *rou;
+
+            SLOUT(slot) = mods[mode][0].mod;
+            if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLOUT(slot)->id))) {
+                ifBLIN_QW0("pgoblin_wident %d", wrt);
+            }
+            if  (!!(ex = SLOUT(slot)->init(exenv, slot))) {
+                ifBLIN_QW0("OUT %s init failed", SLOUT(slot)->name);
+                SLOUT(slot) = NULL;
+                loaded = (0 > ex) ? ex : -ex;
+                goto out;
+            }
+            if  (!(rou = pgoblin_upio(exenv, 0))) {
+                ifBLIN_QX0("No OUT reg 0");
+                loaded = -1;
+                goto out;
+            }
+            MARK_IO_OUT_GO(rou);;
+            ;;  rou->type = slot;
+            ;;  if  (!!pgoblin_out0_open(rou, fileno(stdout))) {
+            ;;      ifBLIN_QW0("[%u]open OUT %c(%u) stdio", slot, rou->nio, pgoblin_regn[rou->nio]);
+            ;;      loaded = -EX_IOERR;
+            ;;      goto out;
+            ;;  }
+            ;;  rou->freeouts = 0;
+            MARK_IO_OUT_WENT(rou);;
+            ++slot;
+        }
+        /*      */
+        if  (!strncmp(mods[mode][0].nm, nm, rs) && !mods[mode][0].nm[rs]) {
+            loaded = PGOBLIN_MODSHIFT;
+            goto out;
+        }
+        /*      */
+        for (int k = 1; !!mods[mode][k].nm && (k < PREPLUGIN); ++k) {
+            if  (!strncmp(mods[mode][k].nm, nm, rs) && !mods[mode][k].nm[rs]) {
+                SLOUT(slot) = mods[mode][k].mod;
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLOUT(slot)->id))) {
+                    ifBLIN_QW0("pgoblin_wident %d", wrt);
+                }
+                if  (!!(ex = SLOUT(slot)->init(exenv, slot))) {
+                    ifBLIN_QW0("OUT %s init failed", SLOUT(slot)->name);
+                    SLOUT(slot) = NULL;
+                } else {
+                    loaded = (int)slot;
+        }   }   }
+        if  (0 > loaded) { /*      */
+            if  (!!(SLOUT(slot) = dload(exenv->options, 0, rs, nm))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLOUT(slot)->id))) {
+                    ifBLIN_QW0("pgoblin_wident %d", wrt);
+                }
+                if  (!!(ex = SLOUT(slot)->init(exenv, slot))) {
+                    ifBLIN_QW0("OUT %s init failed", SLOUT(slot)->name);
+                    SLOUT(slot) = NULL;
+                } else {
+                    loaded = (int)slot;
+        }   }   }
+        break;
+    case 1:
+        /*    ,      */
+        for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.iotablesz) && !!SLIO(slot); ++slot) {
+            ifBLIN_QX7("[%d]=%.*s", slot, (int)rs, SLIO(slot)->name);
+            if  (!strncmp(SLIO(slot)->name, nm, rs) && !SLIO(slot)->name[rs]) {
+                loaded = (int)slot;
+                goto out;
+        }   }
+        if  (slot >= pgoblin.iotablesz) {
+            ifBLIN_QX0("No free slots for IO %s", nm);
+            goto out;
+        }
+        if  (PGOBLIN_MODSHIFT == slot) { /*    ,    */
+            pgoblin_rio *rio;
+
+            SLIO(slot) = mods[mode][0].mod;
+            if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLIO(slot)->id))) {
+                ifBLIN_QW0("pgoblin_wident %d", wrt);
+            }
+            if  (!!(ex = SLIO(slot)->init(exenv, slot))) {
+                ifBLIN_QW0("IO %s init failed", SLIO(slot)->name);
+                SLIO(slot) = NULL;
+                loaded = (0 > ex) ? ex : -ex;
+                goto out;
+            }
+            if  (!(rio = pgoblin_upio(exenv, 0))) {
+                ifBLIN_QX0("No IO reg 0");
+                loaded = -1;
+                goto out;
+            }
+            MARK_IO_OUT_GO(rio);;
+            ;;  rio->type = slot;
+            ;;  if  (!!pgoblin_out0_open(rio, fileno(stdout))) {
+            ;;      ifBLIN_QW0("[%u]open IO %c(%u) stdio", slot, rio->nio, pgoblin_regn[rio->nio]);
+            ;;      ex = -EX_IOERR;
+            ;;      goto out;
+            ;;  }
+            ;;  rio->freeouts = 0;
+            MARK_IO_OUT_WENT(rio);;
+            ++slot;
+        }
+        /*      */
+        if  (!strncmp(mods[mode][0].nm, nm, rs) && !mods[mode][0].nm[rs]) {
+            loaded = PGOBLIN_MODSHIFT;
+            goto out;
+        }
+        /*      */
+        for (int k = 1; !!mods[mode][k].nm && (k < PREPLUGIN); ++k) {
+            if  (!strncmp(mods[mode][k].nm, nm, rs) && !mods[mode][k].nm[rs]) {
+                SLIO(slot) = mods[mode][k].mod;
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLIO(slot)->id))) {
+                    ifBLIN_QW0("pgoblin_wident %d", wrt);
+                }
+                if  (!!(ex = SLIO(slot)->init(exenv, slot))) {
+                    ifBLIN_QW0("IO %s init failed", SLIO(slot)->name);
+                    SLIO(slot) = NULL;
+                } else {
+                    loaded = (int)slot;
+        }   }   }
+        if  (0 > loaded) { /*      */
+            if  (!!(SLIO(slot) = dload(exenv->options, 0, rs, nm))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, SLIO(slot)->id))) {
+                    ifBLIN_QW0("pgoblin_wident %d", wrt);
+                }
+                if  (!!(ex = SLIO(slot)->init(exenv, slot))) {
+                    ifBLIN_QW0("IO %s init failed", SLIO(slot)->name);
+                    SLIO(slot) = NULL;
+                } else {
+                    loaded = (int)slot;
+        }   }   }
+        break;
     case 3:
         for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.dbtablesz) && !!DBASE(slot); ++slot) {
             if  (!strncmp(DBASE(slot)->name, nm, rs) && !DBASE(slot)->name[rs]) {
-                loaded = slot;
+                loaded = (int)slot;
                 goto out;
         }   }
         if  (slot >= pgoblin.dbtablesz) {
@@ -163,10 +313,10 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
         }
         if  (PGOBLIN_MODSHIFT == slot) {
             DBASE(slot) = mods[mode][0].mod;
-            if  (!!(wrt = pgoblin_wident(options, slot, DBASE(slot)->id))) {
+            if  (!!(wrt = pgoblin_wident(exenv->options, slot, DBASE(slot)->id))) {
                 ifBLIN_QW0("pgoblin_wident %d", wrt);
             }
-            if  (!!(ex = DBASE(slot)->init(options, slot))) {
+            if  (!!(ex = DBASE(slot)->init(exenv, slot))) {
                 ifBLIN_QW0("DB %s init failed", DBASE(slot)->name);
                 DBASE(slot) = NULL;
                 loaded = (0 > ex) ? ex : -ex;
@@ -178,34 +328,34 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
             loaded = PGOBLIN_MODSHIFT;
             goto out;
         }
-        for (int k = 1; !!mods[mode][k].nm; ++k) {
+        for (int k = 1; !!mods[mode][k].nm && (k < PREPLUGIN); ++k) {
             if  (!strncmp(mods[mode][k].nm, nm, rs) && !mods[mode][k].nm[rs]) {
                 DBASE(slot) = mods[mode][k].mod;
-                if  (!!(wrt = pgoblin_wident(options, slot, DBASE(slot)->id))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, DBASE(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  (!!(ex = DBASE(slot)->init(options, slot))) {
+                if  (!!(ex = DBASE(slot)->init(exenv, slot))) {
                     ifBLIN_QW0("DB %s init failed", DBASE(slot)->name);
                     DBASE(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         if  (0 > loaded) {
-            if  (!!(DBASE(slot) = dload(options, 3, rs, nm))) {
-                if  (!!(wrt = pgoblin_wident(options, slot, DBASE(slot)->id))) {
+            if  (!!(DBASE(slot) = dload(exenv->options, 3, rs, nm))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, DBASE(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  ((ex = DBASE(slot)->init(options, slot))) {
+                if  (!!(ex = DBASE(slot)->init(exenv, slot))) {
                     ifBLIN_QW0("DB %s init failed", DBASE(slot)->name);
                     DBASE(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         break;
     case 4:
-        for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.jobtabsiz) && !!pgoblin.jobtable[slot]; ++slot) {
-            if  (!strncmp(pgoblin.jobtable[slot]->name, nm, rs) && !pgoblin.jobtable[slot]->name[rs]) {
-                loaded = slot;
+        for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.jobtabsiz) && !!JOBE(slot); ++slot) {
+            if  (!strncmp(JOBE(slot)->name, nm, rs) && !JOBE(slot)->name[rs]) {
+                loaded = (int)slot;
                 goto out;
         }   }
         if  (slot >= pgoblin.jobtabsiz) {
@@ -216,22 +366,22 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
             pgoblin_rjb  *rjb;
             pgoblin_jobl *j;
 
-            pgoblin.jobtable[slot] = mods[mode][0].mod;
-            if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.jobtable[slot]->id))) {
+            JOBE(slot) = mods[mode][0].mod;
+            if  (!!(wrt = pgoblin_wident(exenv->options, slot, JOBE(slot)->id))) {
                 ifBLIN_QW0("pgoblin_wident %d", wrt);
             }
-            if  ((ex = pgoblin.jobtable[slot]->init(options, slot))) {
-                ifBLIN_QW0("JOB %s init failed", pgoblin.jobtable[slot]->name);
-                pgoblin.jobtable[slot] = NULL;
+            if  (!!(ex = pgoblin_jb_init(exenv, slot))) {
+                ifBLIN_QW0("JOB %s init failed", JOBE(slot)->name);
+                JOBE(slot) = NULL;
                 loaded = (0 > ex) ? ex : -ex;
                 goto out;
             }
-            if  (!(rjb = pgoblin_upjob(options, 0))) {
+            if  (!(rjb = pgoblin_upjob(exenv, 0))) {
                 ifBLIN_QX0("No JOB reg 0");
                 loaded = -1;
                 goto out;
             }
-            MARK_R_JOB_GO(0);;
+            MARK_R_JOB_GO(rjb);;
             ;;  rjb->pids = mular_create(MULAR_ZERO, 3, sizeof(pgoblin_jobl), u12);
             ;;  j = mular_add(rjb->pids);
             ;;  j->cmdn = -1;
@@ -241,46 +391,41 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
             ;;  j->cmdn = -1;
             ;;  j->pid = getpid();
             ;;  j->status = PGOBLIN_NOSTATUS;
-            MARK_R_JOB_WENT(0);;
+            MARK_R_JOB_WENT(rjb);;
             ++slot;
         }
         if  (!strncmp(mods[mode][0].nm, nm, rs) && !mods[mode][0].nm[rs]) {
             loaded = PGOBLIN_MODSHIFT;
             goto out;
         }
-        for (int k = 1; !!mods[mode][k].nm; ++k) {
+        for (int k = 1; !!mods[mode][k].nm && (k < PREPLUGIN); ++k) {
             if  (!strncmp(mods[mode][k].nm, nm, rs) && !mods[mode][k].nm[rs]) {
-                pgoblin.jobtable[slot] = mods[mode][k].mod;
-                if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.jobtable[slot]->id))) {
+                JOBE(slot) = mods[mode][k].mod;
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, JOBE(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  ((ex = pgoblin.jobtable[slot]->init(options, slot))) {
-                    ifBLIN_QW0("JOB %s init failed", pgoblin.jobtable[slot]->name);
-                    pgoblin.jobtable[slot] = NULL;
+                if  (!!(ex = pgoblin_jb_init(exenv, slot))) {
+                    ifBLIN_QW0("JOB %s init failed", JOBE(slot)->name);
+                    JOBE(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         if  (0 > loaded) {
-            if  (!!(pgoblin.jobtable[slot] = dload(options, 4, rs, nm))) {
-                if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.jobtable[slot]->id))) {
+            if  (!!(JOBE(slot) = dload(exenv->options, 4, rs, nm))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, JOBE(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  ((ex = pgoblin.jobtable[slot]->init(options, slot))) {
-                    ifBLIN_QW0("DB %s init failed", pgoblin.jobtable[slot]->name);
-                    pgoblin.jobtable[slot] = NULL;
+                if  (!!(ex = pgoblin_jb_init(exenv, slot))) {
+                    ifBLIN_QW0("DB %s init failed", JOBE(slot)->name);
+                    JOBE(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         break;
     case 5:
-        for ( slot = PGOBLIN_MODSHIFT
-            ; (slot < pgoblin.styletbsz) && !!pgoblin.styletable[slot]
-            ; ++slot
-            ) {
-            if  (  !strncmp(pgoblin.styletable[slot]->name, nm, rs)
-                && !pgoblin.styletable[slot]->name[rs]
-                ) {
-                loaded = slot;
+        for (slot = PGOBLIN_MODSHIFT; (slot < pgoblin.styletbsz) && !!STYLS(slot); ++slot) {
+            if  (!strncmp(STYLS(slot)->name, nm, rs) && !STYLS(slot)->name[rs]) {
+                loaded = (int)slot;
                 goto out;
         }   }
         if  (slot >= pgoblin.styletbsz) {
@@ -288,13 +433,13 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
             goto out;
         }
         if  (PGOBLIN_MODSHIFT == slot) {
-            pgoblin.styletable[slot] = mods[mode][0].mod;
-            if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.styletable[slot]->id))) {
+            STYLS(slot) = mods[mode][0].mod;
+            if  (!!(wrt = pgoblin_wident(exenv->options, slot, STYLS(slot)->id))) {
                 ifBLIN_QW0("pgoblin_wident %d", wrt);
             }
-            if  ((ex = pgoblin.styletable[slot]->init(options, slot))) {
-                ifBLIN_QW0("STYLE %s init failed", pgoblin.styletable[PGOBLIN_MODSHIFT]->name);
-                pgoblin.styletable[PGOBLIN_MODSHIFT] = NULL;
+            if  (!!(ex = pgoblin_st_init(exenv, slot))) {
+                ifBLIN_QW0("STYLE %s init failed", STYLS(slot)->name);
+                STYLS(slot) = NULL;
                 loaded = (0 > ex) ? ex : -ex;
                 goto out;
             }
@@ -304,28 +449,28 @@ pgoblin_load(pgoblin_main *options, int mode, const char *nm, int rs) { /**
             loaded = PGOBLIN_MODSHIFT;
             goto out;
         }
-        for (int k = 1; !!mods[mode][k].nm; ++k) {
+        for (int k = 1; !!mods[mode][k].nm && (k < PREPLUGIN); ++k) {
             if  (!strncmp(mods[mode][k].nm, nm, rs) && !mods[mode][k].nm[rs]) {
-                pgoblin.styletable[slot] = mods[mode][k].mod;
-                if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.styletable[slot]->id))) {
+                STYLS(slot) = mods[mode][k].mod;
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, STYLS(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  ((ex = pgoblin.styletable[slot]->init(options, slot))) {
-                    ifBLIN_QW0("STYLE %s init failed", pgoblin.styletable[slot]->name);
-                    pgoblin.styletable[slot] = NULL;
+                if  (!!(ex = pgoblin_st_init(exenv, slot))) {
+                    ifBLIN_QW0("STYLE %s init failed", STYLS(slot)->name);
+                    STYLS(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         if  (0 > loaded) {
-            if  (!!(pgoblin.styletable[slot] = dload(options, 5, rs, nm))) {
-                if  (!!(wrt = pgoblin_wident(options, slot, pgoblin.styletable[slot]->id))) {
+            if  (!!(STYLS(slot) = dload(exenv->options, 5, rs, nm))) {
+                if  (!!(wrt = pgoblin_wident(exenv->options, slot, STYLS(slot)->id))) {
                     ifBLIN_QW0("pgoblin_wident %d", wrt);
                 }
-                if  ((ex = pgoblin.styletable[slot]->init(options, slot))) {
-                    ifBLIN_QW0("DB %s init failed", pgoblin.styletable[slot]->name);
-                    pgoblin.styletable[slot] = NULL;
+                if  (!!(ex = pgoblin_st_init(exenv, slot))) {
+                    ifBLIN_QW0("DB %s init failed", STYLS(slot)->name);
+                    STYLS(slot) = NULL;
                 } else {
-                    loaded = slot;
+                    loaded = (int)slot;
         }   }   }
         break;
     default:;
@@ -335,46 +480,49 @@ out:;
 #   undef blin_internal_flags
 }
  
-pgoblin_main *
+pgoblin_exenv *
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_init(BLIN_flag flags, int outfd) {                          /**
+pgoblin_init(BLIN_flag flags) {                                     /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
 #   define blin_internal_flags (flags & BLIN_MASK)
-    pgoblin_main *options = NULL;
-    pgoblin_rio  *rio;
+    pgoblin_exenv *exenv = NULL;
+    pgoblin_rio   *rio;
 
     blin_ctl(BLIN_CTL_FLAG | BLIN_CTL_FSET, flags);
     ifBLIN_QX2("+ flags=%08X", flags);
-    if  (!(options = calloc(1, sizeof(pgoblin_main)))) {
-        ifBLIN_QX0("no mem for options");
-        errno = ENOMEM;
+    if  (!(exenv = calloc(1, sizeof(pgoblin_exenv)))) {
+        ifBLIN_QW0("no mem for exenv");
         goto out;
     }
-    pgoblin_options = options;
-    options->xtrace = -1;
-    options->flags = flags;
+    exenv->flags = flags & BLIN_MASK;
+    if  (!(exenv->options = calloc(1, sizeof(pgoblin_main)))) {
+        ifBLIN_QW0("no mem for options");
+        free(exenv);
+        exenv = NULL;
+        goto out;
+    }
+    pgoblin_options = exenv->options;
+    exenv->options->xtrace = -1;
+    exenv->options->flags = flags;
     pgoblin_default = flags;
-    strncpy(options->vers, VERS, 4);
-    options->maxtmout = 3600000;
-    strncpy((char*)options->delim, "#", 3);
-    if  (!(rio = pgoblin_upio(options, 0))) {
+    strncpy(exenv->options->vers, VERS, 4);
+    exenv->options->maxtmout = 3600000;
+    strncpy((char*)exenv->options->delim, "#", 3);
+    if  (!(rio = pgoblin_upio(exenv, 0))) {
         ifBLIN_QW0("No IO reg 0");
-        free(options);
-        options = NULL;
+        free(exenv->options);
+        exenv->options = NULL;
+        free(exenv);
+        exenv = NULL;
         goto out;
     }
-    MARK_IO_OUT_GO(0);;
-    ;;  rio->onu = outfd;
-    ;;  rio->wri = (int(*)(int, const void *, size_t))mife_writ;
-    ;;  rio->flags |= PGOBLIN_OUTSET;
-    MARK_IO_OUT_WENT(0);;
 out:;
-    ifBLIN_QX2("- %s", options ? "OK" : "FAIL");
-    return(options);
+    ifBLIN_QX2("- %s", exenv ? "OK" : "FAIL");
+    return(exenv);
 #   undef blin_internal_flags
 }
 
@@ -382,26 +530,30 @@ void
 /**********************************************************************
  **********************************************************************
  **                                                                  **/
-pgoblin_fini(pgoblin_main *options) {                               /**
+pgoblin_fini(pgoblin_exenv *exenv) {                                /**
  **                                                                  **
  **********************************************************************
  **********************************************************************/
-#   define blin_internal_flags (options->flags & BLIN_MASK)
-    int i;
+#   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    u_int i;
 
+    ifBLIN_QX2("+");
     i = pgoblin.dbtablesz;
     if  (i < pgoblin.jobtabsiz) i = pgoblin.jobtabsiz;
     if  (i < pgoblin.styletbsz) i = pgoblin.styletbsz;
-    for (; 0 <= i; --i) {
+    for (; ; --i) {
         if  ((i < pgoblin.styletbsz) && !!pgoblin.styletable[i]) {
-            pgoblin.styletable[i]->fini(options);
+            pgoblin.styletable[i]->fini(exenv);
         }
         if  ((i < pgoblin.dbtablesz) && !!DBASE(i)) {
-            DBASE(i)->fini(options);
+            DBASE(i)->fini(exenv);
         }
         if  ((i < pgoblin.jobtabsiz) && !!pgoblin.jobtable[i]) {
-            pgoblin.jobtable[i]->fini(options);
-    }   }
-    free(options);
+            pgoblin.jobtable[i]->fini(exenv);
+        }
+        if  (!i) break;
+    }
+    free(exenv->options);
+    ifBLIN_QX2("-");
 #   undef blin_internal_flags
 }
