@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2022 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: job_local.c,v 1.55 2022/01/07 13:57:49 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: job_local.c,v 1.76 2025/01/18 23:06:13 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -55,18 +55,21 @@
 static int
 /**********************************************************************
  **                                                                  **/
-init(pgoblin_main *options, int slot) {                             /**
+init(pgoblin_exenv *exenv, u_int slot) {                            /**
  **                                                                  **
  **********************************************************************/
+    (void)exenv;
+    (void)slot;
     return(EX_OK);
 }
 
 static int
 /**********************************************************************
  **                                                                  **/
-fini(pgoblin_main *options) {                                       /**
+fini(pgoblin_exenv *exenv) {                                        /**
  **                                                                  **
  **********************************************************************/
+    (void)exenv;
     return(EX_OK);
 }
 
@@ -76,6 +79,8 @@ static int
 none(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
  **                                                                  **
  **********************************************************************/
+    (void)exenv;
+    (void)rn;
     return(EX_OK);
 }
  
@@ -85,7 +90,9 @@ static int
 jpen(pgoblin_exenv *exenv, pgoblin_nr *rn, void *opt) {             /**
  **                                                                  **
  **********************************************************************/
-    exenv->options->flags |= PGOBLIN_INITED;
+    (void)exenv;
+    (void)rn;
+    (void)opt;
     return(EX_OK);
 }
 
@@ -97,6 +104,7 @@ jest(pgoblin_exenv *exenv, pgoblin_command cmd) {                   /**
  **********************************************************************/
     u_int32_t e;
 
+    (void)exenv;
     if  (cmd == PGOBLIN_fork) e = PGOBLIN_TVFORK; else e = PGOBLIN_TOFORK | PGOBLIN_TOEXIT;
     return(e);
 }
@@ -110,14 +118,13 @@ jxec(pgoblin_exenv *exenv, pgoblin_nr *rn, char **argv) {           /**
     pgoblin_rio *rou;
     pgoblin_rio *rio;
     pgoblin_rio *rin;
-    int          ex = 0;
+    int          ex = EX_SOFTWARE;
 
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
-    ifBLIN_QX4( "+ %d %d %d %d %08X argv %s"
-              , rn[PGO_COUT]
-              , rn[PGO_CCTL]
-              , rn[PGO_CIN]
-              , rn[PGO_CJOB]
+    ifBLIN_QX4( "+ o[%c] i[%c] j[%c] %08X argv %s"
+              , pgoblin_regn[rn[PGO_COUT]]
+              , pgoblin_regn[rn[PGO_CIN]]
+              , pgoblin_regn[rn[PGO_CJOB]]
               , exenv->flags
               , argv ? "OK" : "NULL"
               );
@@ -127,6 +134,7 @@ jxec(pgoblin_exenv *exenv, pgoblin_nr *rn, char **argv) {           /**
     }
     GET_RIO(rou, exenv->options, rn[PGO_COUT]);
     GET_RIO(rin, exenv->options, rn[PGO_CIN]);
+    ex = EX_OK;
     if  (argv && !ex) {
         mife_descriptor *mife = rin->mife;
         int              g;
@@ -146,28 +154,32 @@ jxec(pgoblin_exenv *exenv, pgoblin_nr *rn, char **argv) {           /**
     ;       ex = -EX_OSERR;
     ;       exit(ex);
     ;   }
-    ;   if  (dup2(rou->onu, fileno(stdout)) < 0) {
+    ;   if  (0 > (g = pgoblin_io_fdout(rou))) {
+    ;       ifBLIN_QW0("fdout");
+    ;       ex = g;
+    ;       exit(ex);
+    ;   }
+    ;   if  (0 > dup2(g, fileno(stdout))) {
     ;       ifBLIN_QW0("dup2 #3");
     ;       ex = -EX_OSERR;
     ;       exit(ex);
     ;   }
-    ;   if  (exenv->options->flags & PGOBLIN_FORK_WAIT) {
-    ;       ifBLIN_QX2("sleep %us in fork", exenv->options->flags & PGOBLIN_FORK_WAIT);
-    ;       sleep(exenv->options->flags & PGOBLIN_FORK_WAIT);
-    ;   }
-    ;   for (g = 0; g < pgoblin.regsize; g++) {
-    ;       if  (!!(rio = pgoblin_meio(exenv->options, g))) {
+    ;   for (u_int8_t m = 0; m < pgoblin.regsize; m++) {
+    ;       if  (!!(rio = pgoblin_meio(exenv->options, m))) {
     ;           if  (  rio->mife
                     && (rio->mife->f.fd != fileno(stdin))
                     && (rio->mife->f.fd != fileno(stdout))
                     ) {
-    ;               if  ((rio->flags & PGOBLIN_FREEMIFE) && (ex = mife_fini(rio->mife))) {
+    ;               if  (!!rio->freemife && !!(ex = mife_fini(rio->mife))) {
     ;                   ifBLIN_QW0("mife_fini in exec");
     ;           }   }
-    ;           if  (rio->clo && (rio->onu != fileno(stdin)) && (rio->onu != fileno(stdout))) {
-    ;               if  ((rio->flags & PGOBLIN_FREEOUTS) && (ex = rio->clo(rio->onu))) {
-    ;                   ifBLIN_QW0("clo in exec");
-    ;   }   }   }   }
+    ;           if  (  (m != rn[PGO_COUT])
+                    && (PGOBLIN_MODSHIFT < rio->type)
+                    && !!rio->freeouts
+                    && !!(ex = pgoblin_io_closew(rio))
+                    ) {
+    ;               ifBLIN_QW0("close in exec");
+    ;   }   }   }
     ;   execve(argv[0], argv, exenv->options->envp);
     ;   ex = EX_OSERR;
         ifBLIN_QW0("execve %s", argv[0]);
@@ -181,7 +193,7 @@ out:
 static int
 /**********************************************************************
  **                                                                  **/
-jait(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
+jait(pgoblin_exenv *exenv, pgoblin_rjb *rjb) {                      /**
  **                                                                  **
  **********************************************************************/
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
@@ -191,16 +203,12 @@ jait(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
     int            getlng;
     int            stat;
     pid_t          pid;
-    pgoblin_rjb   *rjb;
     int            exx   = EX_OK;
     int            ex    = EX_OK;
     int            kq;
     pgoblin_jobl  *j;
     size_t         i;
 
-    GET_JOB(rjb, exenv->options, rn[PGO_CJOB]);
-#   undef blin_internal_flags
-#   define blin_internal_flags (rjb->flags & BLIN_MASK)
     if  (rjb) {
         ifBLIN_QX4("+");
         if  (!(setlist = calloc(MULAR_NEXT(rjb->pids), sizeof(struct kevent)))) {
@@ -242,9 +250,9 @@ jait(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
                 ex = EX_OSERR;
                 goto out;
             } else {
-                j = mular_getix(rjb->pids, (int)getone.udata);
+                j = mular_getix(rjb->pids, (size_t)getone.udata);
                 if  ((pid_t)getone.ident == j->pid) {
-                    j->status = getone.data;
+                    j->status = (int)getone.data;
                     if  (0 > (pid = waitpid(j->pid, &stat, WNOHANG))) {
                         ifBLIN_QW0("waitpid %d", j->pid);
                         ex = EX_OSERR;
@@ -252,24 +260,28 @@ jait(pgoblin_exenv *exenv, pgoblin_nr *rn) {                        /**
                     } else if (!pid) {
                         ifBLIN_QX1("waitpid: %d not found", j->pid);
                     }
-                    ifBLIN_QX3( " st=%08X fl=%08X ex=%d exx=%d IFEX=%d EXST=%d"
+                    ifBLIN_QX3( " st=%08X fl=%08X ex=%d exx=%d IFEX=%d EXST=%d EXSIG=%d SIGST=%d"
                               , j->status
                               , rjb->flags
                               , ex
                               , exx
                               , WIFEXITED(j->status)
                               , WEXITSTATUS(j->status)
+                              , WIFSIGNALED(j->status)
+                              , WTERMSIG(j->status)
                               );
-                    if  (  (rjb->flags & PGOBLIN_ERROR_SENSOR)
-                        && WIFEXITED(j->status)
-                        && WEXITSTATUS(j->status)
-                        ) {
-                        exx = WEXITSTATUS(j->status);
-                    }
+                    if  (rjb->flags & PGOBLIN_ERROR_SENSOR) {
+                        if  (  WIFEXITED(j->status)
+                            && WEXITSTATUS(j->status)
+                            ) {
+                            exx = WEXITSTATUS(j->status);
+                        } else if (WIFSIGNALED(j->status)) {
+                            exx = WTERMSIG(j->status) + 256;
+                    }   }
                     if  ((rjb->flags & PGOBLIN_ERROR_SENSIB) && exx) break;
                     ex = EX_OK;
                 } else {
-                    ifBLIN_QW0("Pid %d<>%d in %d", getone.ident, j->pid, (int)j->cmdn);
+                    ifBLIN_QW0("Pid %ld<>%d in %d", (long)getone.ident, j->pid, (int)j->cmdn);
                     ex = EX_OSERR;
         }   }   }
         ifBLIN_QX4("- %d", ex);
@@ -285,11 +297,11 @@ static pgoblin_dir*
 jdir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {          /**
  **                                                                  **
  **********************************************************************/
-    pgoblin_dir *d = NULL;
-    int ex = EX_OK;
-    FTSENT *ent;
-
 #   define blin_internal_flags (exenv->options->flags & BLIN_MASK)
+    pgoblin_dir *d  = NULL;
+    int          ex = EX_OK;
+    FTSENT      *ent;
+
     if  (!dir == !path) {
         ifBLIN_QX0("local: exactly one of dir and path should be");
         ERROUT(EX_DATAERR, EINVAL);
@@ -336,7 +348,7 @@ jdir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {          /**
                 d->f = 0;
                 continue;
             } else fts_set(d->dirp, ent, FTS_SKIP);
-            /* FALLTHRU */
+            BLIN_FALLTHROUGH;
         case FTS_DEFAULT:
         case FTS_F:
             break;
@@ -347,7 +359,7 @@ jdir(pgoblin_exenv *exenv, pgoblin_dir *dir, char *path) {          /**
         case FTS_NS:
             errno = ent->fts_errno;
             ifBLIN_QW1("fts_read NS");
-            /* FALLTHRU */
+            BLIN_FALLTHROUGH;
         case FTS_NSOK:
             ent->fts_statp = NULL;
             break;
