@@ -1,5 +1,5 @@
 /*-
- * Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/
+ * Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -24,14 +24,14 @@
  * SUCH DAMAGE.
  */
 
-#ident "@(#) Copyright (C)2002..2021 @BABOLO http://www.babolo.ru/"
-#ident "@(#) $Id: parser.c,v 1.69 2021/12/26 13:25:31 babolo Exp $"
+#ident "@(#) Copyright (C)2002..2025 @BABOLO http://www.babolo.ru/"
+#ident "@(#) $Id: parser.c,v 1.82 2025/01/25 00:12:13 babolo Exp $"
 
 #define BLIN_COMPAT      4
-#define Bpars_COMPAT     3
+#define Bpars_COMPAT     4
 #define MULAR_COMPAT     0
 #define MIFE_COMPAT      5
-#define PGOBLIN_COMPAT   4
+#define PGOBLIN_COMPAT   5
 #define PGOBLIN_INTERNAL 1
 
 #include <sys/types.h>
@@ -63,7 +63,7 @@ static const char *clasnames[] =
 { "L0", "Ll", "Lb", "Ls", "Ld", "Lo", "Lx"};
 
 static const char class[256] =
-{ L0, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lb, Ll, Ll,  Lo, Ll, Lo, Lo
+{ L0, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lb, Ll, Lo,  Lo, Lo, Lo, Lo
 , Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
 , Lb, Lo, Lo, Ld,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo,  Lo, Lo, Lo, Lo
 , -1, -2, -3, -4,  -5, -6, -7, -8,  -9,-10, Lo, Lo,  Lo, Lo, Lo, Lo
@@ -107,7 +107,7 @@ enum states
 static const char pn[][3] = 
 { "- ", "- ", "- ", "- ", "- ", "- ", "- ", "- "
 , "- ", "- ", "Cp", "- ", "Cb", "Ce", "Cc", "Ci"
-, "Co", "Cl", "- ", "- ", "Cx", "Cy", "- ", "- "
+, "Co", "Cl", "- ", "Cj", "Cx", "Cy", "- ", "- "
 };
 
 #define Cp  0x200000 /*             */
@@ -121,24 +121,22 @@ static const char pn[][3] =
 #define Cx  0x000800 /*                 */
 #define Cy  0x000400 /*                    */
 #define C_state 0xFF /*           */
-#define Cio (Ci | Co)
-#define Cd  (Cb | Ce)
 
 static u_int32_t automa[xx][Lx] =
-/*    \0 L0        \n Ll      \b Lb   Ls       # Ld       * Lo  *    @          */
-{{    Cx|xx,          nl,        bs,        bs,        nq,        bs}/* bg @                   */
-,{    Cx|xx,          nl,        bs,        bs,        bs,        bs}/* bs @                   */
-,{    Cx|xx,          nl,        bs,        bs,        nq,        bs}/* nl @                */
-,{ Co|Cl|xx,  Co|Cl|  cl,  Co|Cl|tb,  Co|Cl|tc,        bs,  Co|Cl|sc}/* nq @ 1           */
-,{ Cl|Cj|xx,     Cl|  cl,     Cl|tb,     Cl|tc,     Cl|sc,     Cl|sc}/* tc                    */
-,{Ci |   xx,          cl,        tb, Cp|    tp,        sc,        sc}/* tb        */
-,{Ci |   xx,          cl,        sc, Cp|    tp,        sc,        sc}/* tp           */
-,{Ci |   xx,          cl,        sc,        sc,        sc,        sc}/* sc                */
-,{Ci |   xx, Cd|   Cc|ml, Cd|Cc| mc, Cd|Cc| mc,        cq, Cd|Cc| mc}/* cl               */
-,{Cio|Cl|xx, Cio|Cl|  ml, Cio|Cl|tb, Cio|Cl|tc, Cd|Cc| mc, Cio|Cl|sc}/* q   1           */
-,{Ci |   xx,    Ce|Cc|ml,    Cc| mc,    Cc| mc,        tq,    Cc| mc}/* ml #             */
-,{Ce|Cc|Ci|xx,  Ce|Cc|ml,    Cc| mc,    Cc| mc, Cc|    mc,    Cc| mc}/* mc #                 */
-,{Cio|Cl|xx, Cio|Cl|  cl, Cio|Cl|tb, Cio|Cl|tc, Cc|    mc, Cio|Cl|sc}/* tq # 1           */
+/*      \0 L0       \n Ll       \b Lb    Ls        # Ld        * Lo  *    @     */
+{{      Cx|xx,         nl,         bs,         bs,         nq,         bs}/* bg @              */
+,{      Cx|xx,         nl,         bs,         bs,         bs,         bs}/* bs @              */
+,{      Cx|xx,         nl,         bs,         bs,         nq,         bs}/* nl @           */
+,{   Co|Cl|xx,   Co|Cl|cl,   Co|Cl|tb,   Co|Cl|tc,         bs,   Co|Cl|sc}/* nq @ 1      */
+,{   Cl|Cj|xx,      Cl|cl,      Cl|tb,      Cl|tc,      Cl|sc,      Cl|sc}/* tc               */
+,{Ci|      xx,         cl,         tb,   Cp|   tp,         sc,         sc}/* tb       */
+,{Ci|      xx,         cl,         sc,   Cp|   tp,         sc,         sc}/* tp      */
+,{Ci|      xx,         cl,         sc,         sc,         sc,         sc}/* sc           */
+,{Ci|      xx,Cb|Ce|Cc|ml,Cb|Ce|Cc|mc,Cb|Ce|Cc|mc,         cq,Cb|Ce|Cc|mc}/* cl          */
+,{Ci|Co|Cl|xx,Ci|Co|Cl|ml,Ci|Co|Cl|tb,Ci|Co|Cl|tc,Cb|Ce|Cc|mc,Ci|Co|Cl|sc}/* q   1      */
+,{Ci|      xx,   Ce|Cc|ml,      Cc|mc,      Cc|mc,         tq,      Cc|mc}/* ml #        */
+,{Ci|Ce|Cc|xx,   Ce|Cc|ml,      Cc|mc,      Cc|mc,      Cc|mc,      Cc|mc}/* mc #            */
+,{Ci|Co|Cl|xx,Ci|Co|Cl|cl,Ci|Co|Cl|tb,Ci|Co|Cl|tc,      Cc|mc,Ci|Co|Cl|sc}/* tq # 1      */
 };                                                                      /* #                  */
 
 static int
@@ -156,15 +154,14 @@ placmd( pgoblin_main *options                                       /**/
  **                                                                  **
  **********************************************************************/
     pgoblin_tuple *command;
-    int            ex = EX_OK;
-    int            i;
+    int            ex     = EX_OK;
 
 #   define blin_internal_flags (options->flags & BLIN_MASK)
     ifBLIN_QX3( "[%"BLIN_D"]=%03X=%s%.*s(%016"BLIN_O"X)"
               , MULAR_NEXT(pgm->code)
               , r_cmd
               , pgoblin.a[r_cmd].str
-              , 9 - strlen(pgoblin.a[r_cmd].str), "         "
+              , 9 - (int)strlen(pgoblin.a[r_cmd].str), "         "
               , r_parm
               );
     if  (!r_cmd) {
@@ -186,8 +183,8 @@ placmd( pgoblin_main *options                                       /**/
 
         cc = 0;
         *localflags &= ~PGOBLIN_IGNORE;
-        for (i = 0; i < pgoblin.symparam; i++) {
-            if  (0 <= (b = pgoblin_breg(r_cmd, i))) {
+        for (pgoblin_nr r = 0; r < pgoblin.symparam; r++) {
+            if  (0 <= (b = pgoblin_breg(r_cmd, r))) {
                 u_int32_t y = pgoblin.binparams[b];
 
                 cc &= ~(PGOBLIN_ARGMASK << y);
@@ -212,7 +209,7 @@ placmd( pgoblin_main *options                                       /**/
                     command->lit = r_beg;
                 }
                 if  (command->lit) {
-                        ifBLIN_QX3(">%s~", command->lit);
+                    ifBLIN_QX3(">%s~", command->lit);
     }   }   }   }
 out:
     return(ex);
@@ -235,6 +232,7 @@ int
  ********************************************************************************************
  **                                                                                        **/
 pgoblin_parser( pgoblin_main  *options                                                    /**/
+              , void          *tomem                                                      /**/
               , u_char        *text                                                       /**/
               , ssize_t        txlen                                                      /**/
               , pgoblin_prog **ppgm                                                       /**/
@@ -244,19 +242,19 @@ pgoblin_parser( pgoblin_main  *options
  **                                                                                        **
  ********************************************************************************************
  ********************************************************************************************/
-    size_t         sz[]    = {512, 512, 512};
-    u_char        *literal = NULL;
-    char          *r_beg   = NULL;
-    u_char        *r_end   = NULL;
     u_int32_t      localflags;
-    int            ex      = EX_OK;
-    pgoblin_prog  *pgm     = NULL;
     u_int32_t      control;
+    u_char        *literal = NULL;
     u_int64_t      r_parm;
     ssize_t        offset;
     enum states    state;
     u_int32_t      r_cmd;
     u_int32_t      r_lex;
+    char          *r_beg   = NULL;
+    u_char        *r_end   = NULL;
+    pgoblin_prog  *pgm     = NULL;
+    size_t         sz[]    = {512, 512, 512};
+    int            ex      = EX_OK;
     const u_char  *qp;
     const u_char **qq;
     int            rs;
@@ -264,10 +262,12 @@ pgoblin_parser( pgoblin_main  *options
     u_char         s;
 
 #   define blin_internal_flags (options->flags & BLIN_MASK)
-    ifBLIN_QX2( "+ options %s, ppgm %s txt %"BLIN_D
+    ifBLIN_QX2( "+ options %s, ppgm %s txt %"BLIN_D"=%.*s~"
               , options ? "OK" : "NULL"
               , ppgm ? "OK" : "NULL"
               , txlen
+              , (int)txlen
+              , text
               );
     if  (!text && txlen) {
         ifBLIN_QX0("No pgoblin program text");
@@ -278,7 +278,7 @@ pgoblin_parser( pgoblin_main  *options
     } else if (!!(pgm = *ppgm)) {
         ifBLIN_QX2("pgm %s", pgm ? "OK" : "NULL");
         if  (strncmp(pgm->id, "#pGoblin-" VERS, PGOBLIN_STRING_ID_TST)) {
-            ifBLIN_QX0("Illegal pgoblin program");
+            ifBLIN_QX0("Illegal pgoblin program id=%.*s", PGOBLIN_STRING_ID_TST, pgm->id);
             ERROUT(EX_DATAERR, EPROGMISMATCH);
         }
     } else if (!(pgm = calloc(1, sizeof(pgoblin_prog)))) {
@@ -303,6 +303,17 @@ pgoblin_parser( pgoblin_main  *options
         goto out;
     }
     ifBLIN_QX3("pgm@%"BLIN_D, MULAR_NEXT(pgm->code));
+    if  (!!tomem) {
+        void *a = pgm->tofree;
+
+        if  (!(pgm->tofree = malloc(sizeof(pgoblin_list)))) {
+            ifBLIN_QW0("Malloc failed for pgm->tofree");
+            ex = EX_OSERR;
+            goto out;
+        }
+        pgm->tofree->next = a;
+        pgm->tofree->mem = tomem;
+    }
     blin_stateheader(BLIN_5STO24G, pn);
     qp = &s;
     qq = &qp;
@@ -327,12 +338,13 @@ pgoblin_parser( pgoblin_main  *options
                       , statenames
                       , clasnames[(u_char)p]
                       , (char*)text
-                      , offset
+                      , (size_t)offset
                       , control
                       , state
-                      , r_lex
+                      , (u_int32_t)(literal - text) // r_lex
                       , rs & 0xFF
                       );
+
         if  (control & Cp) r_parm |= (~(u_int64_t)(class[s])) << (rs += PGOBLIN_BPARG);
         if  (control & Cb && !r_beg) r_beg = (char*)(literal = &text[offset]);
         if  (control & Ce && r_beg) r_end = literal;
@@ -376,8 +388,36 @@ pgoblin_parser( pgoblin_main  *options
             ex = EX_DATAERR;
     }   }
 out:
-    *ppgm = pgm;
-    ifBLIN_QX2("- %d pgm %s", ex, pgm ? "OK" : "NULL");
+    if  (!!ppgm) *ppgm = pgm;
+    ifBLIN_QX2("- %d pgm %s", ex, !!pgm ? "OK" : "NULL");
     return(ex);
 #   undef blin_internal_flags
 }
+
+void
+/**********************************************************************
+ **********************************************************************
+ **                                                                  **/
+pgoblin_dexenv(pgoblin_exenv **exenv) {                             /**
+ **                                                                  **
+ **********************************************************************
+ **********************************************************************/
+    BLIN_flag        flags;
+
+    if  (!!exenv && !!*exenv) {
+        flags = (*exenv)->flags;
+#   define blin_internal_flags (flags & BLIN_MASK)
+        (void)blin_internal_flags;
+        if  ((*exenv)->pgm) {
+            if  (!!(*exenv)->pgm->code) mular_destroy((*exenv)->pgm->code);
+            for (pgoblin_list *a = (*exenv)->pgm->tofree; !!a;) {
+                pgoblin_list *b = a->next;
+                free(a);
+                a = b;
+            }
+            free((*exenv)->pgm);
+        }
+        free(*exenv);
+        *exenv = NULL;
+#   undef blin_internal_flags
+}   }
